@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Build Utility
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 using System;
@@ -156,8 +186,6 @@ namespace BuildUtil
 			Win32BuildUtil.ExecCommand(Env.ExeFileName, string.Format("/CMD:CopyRelease"));
 
 #if !BU_SOFTETHER
-			Win32BuildUtil.ExecCommand(Env.ExeFileName, string.Format("/CMD:MakeOpenSource"));
-
 			Win32BuildUtil.ExecCommand(Env.ExeFileName, string.Format("/CMD:MakeSoftEtherDir"));
 
 			if (vl["SEVPN"].BoolValue)
@@ -165,6 +193,8 @@ namespace BuildUtil
 				// Build SEVPN
 				Win32BuildUtil.ExecCommand(Paths.CmdFileName, string.Format("/C \"{0}\"", Path.Combine(Paths.SoftEtherBuildDir, @"Main\BuildAll.cmd")));
 			}
+
+			Win32BuildUtil.ExecCommand(Env.ExeFileName, string.Format("/CMD:MakeOpenSource"));
 #endif
 
 			DateTime end = Time.NowDateTime;
@@ -334,6 +364,8 @@ namespace BuildUtil
 
 				tmp = Str.ReplaceStr(tmp, "\\\\", "\\");
 
+				tmp = Str.ReplaceStr(tmp, " ", "_");
+
 				w.WriteLine("mkdir \"{1}\\{0}\"", Path.GetDirectoryName(tmp), cddir);
 				w.WriteLine("copy /b /y \"{2}\\{0}\" \"{3}\\{1}\"", IO.GetRelativeFileName(file, filesReleaseDir), tmp, baseName, cddir);
 				w.WriteLine();
@@ -478,9 +510,64 @@ namespace BuildUtil
 			txt.WriteLine("*");
 			txt.WriteLine();
 
+			string src_bindir = Path.Combine(Paths.BaseDirName, "bin");
+			string vpnsmgr_zip_filename_relative = @"Windows\Admin_Tools\VPN_Server_Manager_and_Command-line_Utility_Package\";
+			vpnsmgr_zip_filename_relative += 
+#if BU_SOFTETHER
+				"softether-" + 
+#endif	// BU_SOFTETHER
+			string.Format("vpn_admin_tools-v{0}.{1:D2}-{2}-{3}-{4:D4}.{5:D2}.{6:D2}-win32.zip",
+				version / 100, version % 100, build, name,
+				date.Year, date.Month, date.Day);
+
+			string vpnsmgr_zip_filename_full = Path.Combine(Path.Combine(publicDir, cddir), vpnsmgr_zip_filename_relative);
+
+			ZipPacker zip = new ZipPacker();
+			zip.AddFileSimple("vpnsmgr.exe", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, "vpnsmgr.exe")), true);
+			zip.AddFileSimple("vpncmd.exe", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, "vpncmd.exe")), true);
+			zip.AddFileSimple("hamcore.se2", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, @"BuiltHamcoreFiles\hamcore_win32\hamcore.se2")), true);
+			zip.AddFileSimple("ReadMeFirst_License.txt", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, @"hamcore\eula.txt")), true);
+			zip.AddFileSimple("ReadMeFirst_Important_Notices_ja.txt", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, @"hamcore\warning_ja.txt")), true);
+			zip.AddFileSimple("ReadMeFirst_Important_Notices_en.txt", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, @"hamcore\warning_en.txt")), true);
+			zip.AddFileSimple("ReadMeFirst_Important_Notices_cn.txt", DateTime.Now, FileAttributes.Normal,
+				IO.ReadFile(Path.Combine(src_bindir, @"hamcore\warning_cn.txt")), true);
+			zip.Finish();
+			byte[] zip_data = zip.GeneratedData.Read();
+			IO.MakeDirIfNotExists(Path.GetDirectoryName(vpnsmgr_zip_filename_full));
+			IO.SaveFile(vpnsmgr_zip_filename_full, zip_data);
+
+			// ZIP package for VPN Server Manager GUI
+			txt.WriteLine("FILENAME\t" + Str.ReplaceStr(vpnsmgr_zip_filename_relative, @"\", "/"));
+#if BU_SOFTETHER
+			txt.WriteLine("DESCRIPTION\t" + "ZIP Package of vpnsmgr.exe and vpncmd.exe (without installers)");
+			txt.WriteLine("CATEGORY\t" + "SoftEther VPN (Freeware)");
+			txt.WriteLine("PRODUCT\t" + "SoftEther VPN Server Manager for Windows, SoftEther VPN Command-Line Admin Utility (vpncmd)");
+#else	// BU_SOFTETHER
+			txt.WriteLine("DESCRIPTION\t" + "ZIP Package of vpnsmgr.exe and vpncmd.exe (without installers)");
+			txt.WriteLine("CATEGORY\t" + "PacketiX VPN (Commercial)");
+			txt.WriteLine("PRODUCT\t" + "PacketiX VPN Server Manager for Windows, PacketiX VPN Command-Line Admin Utility (vpncmd)");
+#endif	// BU_SOFTETHER
+			txt.WriteLine("OS\t" + "Windows (.zip package without installers)");
+			txt.WriteLine("OSLIST\t" + OSList.Windows.OSSimpleList);
+			txt.WriteLine("CPU\t" + "Intel (x86 and x64)");
+			txt.WriteLine("VERSION\t" + BuildHelper.VersionIntToString(version));
+			txt.WriteLine("BUILD\t" + build.ToString());
+			txt.WriteLine("VERSTR\t" + name);
+			txt.WriteLine("DATE\t" + Str.DateTimeToStrShortWithMilliSecs(date));
+			txt.WriteLine("LANGUAGE\t" + "English, Japanese, Simplified Chinese");
+			txt.WriteLine("*");
+			txt.WriteLine();
+
 			IO.MakeDirIfNotExists(Path.Combine(publicDir, cddir));
 			File.WriteAllText(Path.Combine(Path.Combine(publicDir, cddir), "files.txt"), txt.ToString(), Str.Utf8Encoding);
 
+
 			// Execution of batch file
 			string old_cd = Environment.CurrentDirectory;
 
@@ -839,6 +926,19 @@ namespace BuildUtil
 			return 0;
 		}
 
+		// Driver package build
+		// Win32 build
+		[ConsoleCommandMethod(
+			"Builds the driver package.",
+			"BuildDriverPackage",
+			"Builds the driver package.")]
+		static int BuildDriverPackage(ConsoleService c, string cmdName, string str)
+		{
+			Win32BuildUtil.MakeDriverPackage();
+
+			return 0;
+		}
+
 		// Win32 build
 		[ConsoleCommandMethod(
 			"Builds all executable files for win32 and HamCore for all OS.",
@@ -1072,14 +1172,16 @@ namespace BuildUtil
 			{
 				new ConsoleParam("[targetFileName]", ConsoleService.Prompt, "Target Filename: ", ConsoleService.EvalNotEmpty, null),
 				new ConsoleParam("OUT", ConsoleService.Prompt, "Dst Filename: ", ConsoleService.EvalNotEmpty, null),
+				new ConsoleParam("PRODUCT"),
 				new ConsoleParam("RC"),
 			};
 			ConsoleParamValueList vl = c.ParseCommandList(cmdName, str, args);
 
 			string targetFilename = vl.DefaultParam.StrValue;
 			string outFilename = vl["OUT"].StrValue;
+			string product_name = vl["PRODUCT"].StrValue;
 
-			Win32BuildUtil.GenerateVersionInfoResource(targetFilename, outFilename, vl["RC"].StrValue);
+			Win32BuildUtil.GenerateVersionInfoResource(targetFilename, outFilename, vl["RC"].StrValue, product_name);
 
 			return 0;
 		}
@@ -1220,6 +1322,8 @@ namespace BuildUtil
 				new ConsoleParam("DEST"),
 				new ConsoleParam("COMMENT", ConsoleService.Prompt, "Comment: ", ConsoleService.EvalNotEmpty, null),
 				new ConsoleParam("KERNEL"),
+				new ConsoleParam("CERTID"),
+				new ConsoleParam("SHAMODE"),
 			};
 			ConsoleParamValueList vl = c.ParseCommandList(cmdName, str, args);
 
@@ -1232,14 +1336,13 @@ namespace BuildUtil
 			string comment = vl["COMMENT"].StrValue;
 			bool kernel = vl["KERNEL"].BoolValue;
 
-			CodeSign.SignFile(destFileName, srcFileName, comment, kernel);
+			int certid = vl["CERTID"].IntValue;
+			int shamode = vl["SHAMODE"].IntValue;
+
+			CodeSign.SignFile(destFileName, srcFileName, comment, kernel, certid, shamode);
 
 			return 0;
 		}
 	}
 }
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
