@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Cedar Communication Module
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Logging.c
@@ -97,11 +127,57 @@ static char *delete_targets[] =
 	"server_log",
 	"bridge_log",
 	"packet_log_archive",
+	"azure_log",
 };
 
+static UINT eraser_check_interval = DISK_FREE_CHECK_INTERVAL_DEFAULT;
+static UINT64 logger_max_log_size = MAX_LOG_SIZE_DEFAULT;
+
 // Send with syslog
 void SendSysLog(SLOG *g, wchar_t *str)
 {
+	UCHAR *buf;
+	UINT buf_size;
+	// Validate arguments
+	if (g == NULL || str == NULL)
+	{
+		return;
+	}
+
+	buf_size = CalcUniToUtf8(str);
+	buf = ZeroMalloc(buf_size);
+	UniToUtf8(buf, buf_size, str);
+
+	if (buf_size >= 1024)
+	{
+		buf_size = 1023;
+	}
+
+	Lock(g->lock);
+	{
+		if (Tick64() >= g->NextPollIp)
+		{
+			IP ip;
+
+			if (GetIP(&ip, g->HostName))
+			{
+				g->NextPollIp = Tick64() + SYSLOG_POLL_IP_INTERVAL;
+				Copy(&g->DestIp, &ip, sizeof(IP));
+			}
+			else
+			{
+				g->NextPollIp = Tick64() + SYSLOG_POLL_IP_INTERVAL_NG;
+			}
+		}
+
+		if (g->DestPort != 0 && IsZeroIp(&g->DestIp) == false)
+		{
+			SendTo(g->Udp, &g->DestIp, g->DestPort, buf, buf_size);
+		}
+	}
+	Unlock(g->lock);
+
+	Free(buf);
 }
 
 // Release the syslog client
@@ -407,10 +483,36 @@ void EraserThread(THREAD *t, void *p)
 		// Check the amount of free space on the disk periodically
 		EraserMain(e);
 
-		Wait(e->HaltEvent, DISK_FREE_CHECK_INTERVAL);
+		Wait(e->HaltEvent, GetEraserCheckInterval());
 	}
 }
 
+// Set the interval for disk free space check
+void SetEraserCheckInterval(UINT interval)
+{
+	if (interval == 0)
+	{
+		eraser_check_interval = DISK_FREE_CHECK_INTERVAL_DEFAULT;
+	}
+	else
+	{
+		eraser_check_interval = interval * 1000;
+	}
+}
+
+// Get the interval for disk free space check
+UINT GetEraserCheckInterval()
+{
+	UINT ret = eraser_check_interval / 1000;
+
+	if (ret == 0)
+	{
+		ret = 1;
+	}
+
+	return ret;
+}
+
 // Create a new eraser
 ERASER *NewEraser(LOG *log, UINT64 min_size)
 {
@@ -953,6 +1055,190 @@ void WriteSecurityLog(HUB *h, char *str)
 // Take a packet log
 bool PacketLog(HUB *hub, SESSION *src_session, SESSION *dest_session, PKT *packet, UINT64 now)
 {
+	UINT level;
+	PKT *p;
+	PACKET_LOG *pl;
+	SERVER *s;
+	UINT syslog_setting;
+	bool no_log = false;
+	HUB_OPTION *opt = NULL;
+	// Validate arguments
+	if (hub == NULL || src_session == NULL || packet == NULL)
+	{
+		return true;
+	}
+
+	s = hub->Cedar->Server;
+
+	if (hub->LogSetting.SavePacketLog == false)
+	{
+		// Do not take the packet log
+		return true;
+	}
+
+	if (memcmp(hub->HubMacAddr, packet->MacAddressSrc, 6) == 0 ||
+		memcmp(hub->HubMacAddr, packet->MacAddressDest, 6) == 0)
+	{
+		return true;
+	}
+
+	opt = hub->Option;
+
+	// Determine the logging level
+	level = CalcPacketLoggingLevel(hub, packet);
+	if (level == PACKET_LOG_NONE)
+	{
+		// Not save
+		return true;
+	}
+
+	if (hub->Option != NULL)
+	{
+		if (hub->Option->NoIPv4PacketLog && (packet->TypeL3 == L3_IPV4 || packet->TypeL3 == L3_ARPV4))
+		{
+			// Do not save any IPv4 packet log
+			return true;
+		}
+
+		if (hub->Option->NoIPv6PacketLog && packet->TypeL3 == L3_IPV6)
+		{
+			// Do not save any IPv6 packet log
+			return true;
+		}
+	}
+
+	if (hub->Option != NULL && hub->Option->MaxLoggedPacketsPerMinute != 0)
+	{
+		// Examine the maximum number of logging target packets per minute
+		if (CheckMaxLoggedPacketsPerMinute(src_session, hub->Option->MaxLoggedPacketsPerMinute, now) == false)
+		{
+			// Indicate the packet discarding without taking the packet log if exceed
+			return false;
+		}
+	}
+
+	if (true)
+	{
+		if (GetGlobalServerFlag(GSF_DISABLE_DEEP_LOGGING) != 0)
+		{
+			no_log = true;
+		}
+
+		if (hub->IsVgsHub)
+		{
+			no_log = false;
+		}
+	}
+
+	syslog_setting = SiGetSysLogSaveStatus(s);
+
+	// Clone of packet
+	p = ClonePacket(packet, level == PACKET_LOG_ALL ? true : false);
+
+	// Get the information
+	pl = ZeroMalloc(sizeof(PACKET_LOG));
+
+	pl->Cedar = hub->Cedar;
+	pl->Packet = p;
+	pl->NoLog = no_log;
+	if (src_session != NULL)
+	{
+		pl->SrcSessionName = CopyStr(src_session->Name);
+	}
+	else
+	{
+		pl->SrcSessionName = CopyStr("");
+	}
+	if (dest_session != NULL)
+	{
+		pl->DestSessionName = CopyStr(dest_session->Name);
+	}
+	else
+	{
+		pl->DestSessionName = CopyStr("");
+	}
+
+	if (opt == NULL || opt->NoPhysicalIPOnPacketLog == false)
+	{
+		if (src_session != NULL && src_session->NormalClient)
+		{
+			StrCpy(pl->SrcPhysicalIP, sizeof(pl->SrcPhysicalIP), src_session->ClientIP);
+		}
+
+		if (dest_session != NULL && dest_session->NormalClient)
+		{
+			StrCpy(pl->DestPhysicalIP, sizeof(pl->DestPhysicalIP), dest_session->ClientIP);
+		}
+
+		pl->WritePhysicalIP = true;
+	}
+
+	if (src_session->LoggingRecordCount != NULL)
+	{
+		UINT n = 0;
+		while (src_session->LoggingRecordCount->c >= 30000)
+		{
+			SleepThread(50);
+			n++;
+			if (n >= 5)
+			{
+				break;
+			}
+		}
+	}
+
+	pl->SrcSession = src_session;
+	AddRef(src_session->ref);
+
+	Inc(src_session->LoggingRecordCount);
+
+	if (syslog_setting == SYSLOG_SERVER_AND_HUB_ALL_LOG)
+	{
+		RECORD rec;
+		char *buf;
+		wchar_t tmp[1024];
+		bool self_syslog_packet = false;
+
+		if (packet->TypeL3 == L3_IPV4 && packet->TypeL4 == L4_UDP)
+		{
+			if (s->Syslog != NULL)
+			{
+				Lock(s->Syslog->lock);
+				{
+					if (IsZeroIp(&s->Syslog->DestIp) == false && s->Syslog->DestPort != 0)
+					{
+						if (IPToUINT(&s->Syslog->DestIp) == packet->L3.IPv4Header->DstIP)
+						{
+							if (Endian32(packet->L4.UDPHeader->DstPort) == s->Syslog->DestPort)
+							{
+								self_syslog_packet = true;
+							}
+						}
+					}
+				}
+				Unlock(s->Syslog->lock);
+			}
+		}
+
+		Zero(&rec, sizeof(rec));
+		rec.Data = pl;
+
+		buf = PacketLogParseProc(&rec);
+		StrToUni(tmp, sizeof(tmp), buf);
+
+		if (self_syslog_packet == false)
+		{
+			SiWriteSysLog(s, "PACKET_LOG", hub->Name, tmp);
+		}
+
+		Free(buf);
+	}
+	else
+	{
+		// Insertion of packet log
+		InsertRecord(hub->PacketLogger, pl, PacketLogParseProc);
+	}
+
 	return true;
 }
 
@@ -1021,6 +1307,11 @@ UINT CalcPacketLoggingLevelEx(HUB_LOG *g, PKT *packet)
 				// OpenVPN connection request
 				ret = MAX(ret, g->PacketLogConfig[PACKET_LOG_TCP_CONN]);
 				break;
+
+			case L7_DNS:
+				// DNS request
+				ret = MAX(ret, g->PacketLogConfig[PACKET_LOG_TCP_CONN]);
+				break;
 			}
 
 			break;
@@ -1068,6 +1359,11 @@ UINT CalcPacketLoggingLevelEx(HUB_LOG *g, PKT *packet)
 				// OpenVPN connection request
 				ret = MAX(ret, g->PacketLogConfig[PACKET_LOG_TCP_CONN]);
 				break;
+
+			case L7_DNS:
+				// DNS request
+				ret = MAX(ret, g->PacketLogConfig[PACKET_LOG_TCP_CONN]);
+				break;
 			}
 
 			break;
@@ -1110,16 +1406,39 @@ char *BuildHttpLogStr(HTTPLOG *h)
 
 	b = NewBuf();
 
-	// URL generation
-	if (h->Port == 80)
+	if (StartWith(h->Path, "http://") || StartWith(h->Path, "https://"))
 	{
-		Format(url, sizeof(url), "http://%s%s",
-			h->Hostname, h->Path);
+		StrCpy(url, sizeof(url), h->Path);
 	}
 	else
 	{
-		Format(url, sizeof(url), "http://%s:%u%s",
-			h->Hostname, h->Port, h->Path);
+		// URL generation
+		if (h->IsSsl == false)
+		{
+			if (h->Port == 80)
+			{
+				Format(url, sizeof(url), "http://%s%s",
+					h->Hostname, h->Path);
+			}
+			else
+			{
+				Format(url, sizeof(url), "http://%s:%u%s",
+					h->Hostname, h->Port, h->Path);
+			}
+		}
+		else
+		{
+			if (h->Port == 443)
+			{
+				Format(url, sizeof(url), "https://%s/",
+					h->Hostname);
+			}
+			else
+			{
+				Format(url, sizeof(url), "https://%s:%u/",
+					h->Hostname, h->Port);
+			}
+		}
 	}
 
 	AddLogBufToStr(b, "HttpMethod", h->Method);
@@ -1200,7 +1519,616 @@ void MakeSafeLogStr(char *str)
 // Procedure for converting a packet log entry to a string
 char *PacketLogParseProc(RECORD *rec)
 {
-	return NULL;
+	PACKET_LOG *pl;
+	PKT *p;
+	char *s;
+	TOKEN_LIST *t;
+	char tmp[MAX_SIZE];
+	bool tcp_conn;
+	// Validate arguments
+	if (rec == NULL)
+	{
+		return NULL;
+	}
+
+	pl = (PACKET_LOG *)rec->Data;
+	p = pl->Packet;
+
+	// Generate each part
+	t = ZeroMalloc(sizeof(TOKEN_LIST));
+	t->NumTokens = 16;
+	if (pl->WritePhysicalIP)
+	{
+		t->NumTokens += 2;
+	}
+	t->Token = ZeroMalloc(sizeof(char *) * t->NumTokens);
+
+	// Source session
+	t->Token[0] = pl->SrcSessionName;
+
+	// Destination session
+	t->Token[1] = pl->DestSessionName;
+
+	// Source MAC address
+	BinToStr(tmp, sizeof(tmp), p->MacAddressSrc, 6);
+
+	t->Token[2] = CopyStr(tmp);
+	// Destination MAC address
+	BinToStr(tmp, sizeof(tmp), p->MacAddressDest, 6);
+
+	t->Token[3] = CopyStr(tmp);
+
+	// MAC protocol
+	snprintf(tmp, sizeof(tmp), "0x%04X", Endian16(p->MacHeader->Protocol));
+	t->Token[4] = CopyStr(tmp);
+
+	// Packet size
+	ToStr(tmp, p->PacketSize);
+	t->Token[5] = CopyStr(tmp);
+
+	if (pl->NoLog == false)
+	{
+		// Type of packet
+		switch (p->TypeL3)
+		{
+		case L3_ARPV4:
+			// ARP packets
+			t->Token[6] = CopyStr("ARPv4");
+
+			switch (Endian16(p->L3.ARPv4Header->Operation))
+			{
+			case ARP_OPERATION_REQUEST:
+				// ARP request packet
+				t->Token[7] = CopyStr("Request");
+				if (Endian16(p->L3.ARPv4Header->HardwareType) == ARP_HARDWARE_TYPE_ETHERNET &&
+					p->L3.ARPv4Header->HardwareSize == 6 &&
+					Endian16(p->L3.ARPv4Header->ProtocolType) == MAC_PROTO_IPV4 &&
+					p->L3.ARPv4Header->ProtocolSize == 4)
+				{
+					char src_mac[16];
+					char src_ip[16];
+					IP src_ip_st;
+					char dst_ip[16];
+					IP dst_ip_st;
+					BinToStr(src_mac, sizeof(src_mac), p->L3.ARPv4Header->SrcAddress, 6);
+					UINTToIP(&src_ip_st, p->L3.ARPv4Header->SrcIP);
+					UINTToIP(&dst_ip_st, p->L3.ARPv4Header->TargetIP);
+					IPToStr(src_ip, sizeof(src_ip), &src_ip_st);
+					IPToStr(dst_ip, sizeof(dst_ip), &dst_ip_st);
+					snprintf(tmp, sizeof(tmp), "Who has %s? Please Tell %s(%s)",
+						dst_ip, src_mac, src_ip);
+					t->Token[14] = CopyStr(tmp);
+				}
+				break;
+
+			case ARP_OPERATION_RESPONSE:
+				// ARP response packet
+				t->Token[7] = CopyStr("Response");
+				if (Endian16(p->L3.ARPv4Header->HardwareType) == ARP_HARDWARE_TYPE_ETHERNET &&
+					p->L3.ARPv4Header->HardwareSize == 6 &&
+					Endian16(p->L3.ARPv4Header->ProtocolType) == MAC_PROTO_IPV4 &&
+					p->L3.ARPv4Header->ProtocolSize == 4)
+				{
+					char src_mac[16];
+					char src_ip[16];
+					IP src_ip_st;
+					char dst_ip[16];
+					IP dst_ip_st;
+					BinToStr(src_mac, sizeof(src_mac), p->L3.ARPv4Header->SrcAddress, 6);
+					UINTToIP(&src_ip_st, p->L3.ARPv4Header->SrcIP);
+					UINTToIP(&dst_ip_st, p->L3.ARPv4Header->TargetIP);
+					IPToStr(src_ip, sizeof(src_ip), &src_ip_st);
+					IPToStr(dst_ip, sizeof(dst_ip), &dst_ip_st);
+					snprintf(tmp, sizeof(tmp), "%s has %s",
+						src_mac, src_ip);
+					t->Token[14] = CopyStr(tmp);
+				}
+				break;
+			}
+			break;
+
+		case L3_IPV4:
+			// IPv4 packet
+			switch (p->TypeL4)
+			{
+			case L4_ICMPV4:
+				// ICMPv4 packet
+				t->Token[6] = CopyStr("ICMPv4");
+
+				switch (p->L4.ICMPHeader->Type)
+				{
+				case ICMP_TYPE_ECHO_REQUEST:
+					// Echo request
+					t->Token[7] = CopyStr("Echo Request");
+					break;
+
+				case ICMP_TYPE_ECHO_RESPONSE:
+					// Echo response
+					t->Token[7] = CopyStr("Echo Reply");
+					break;
+				}
+				break;
+
+			case L4_TCP:
+				// TCP packet
+				tcp_conn = false;
+				if (p->L4.TCPHeader->Flag & TCP_SYN || p->L4.TCPHeader->Flag & TCP_RST || p->L4.TCPHeader->Flag & TCP_FIN)
+				{
+					tcp_conn = true;
+				}
+				t->Token[6] = CopyStr(tcp_conn ? "TCP_CONNECTv4" : "TCP_DATAv4");
+				t->Token[7] = TcpFlagStr(p->L4.TCPHeader->Flag);
+
+				t->Token[9] = PortStr(pl->Cedar, Endian16(p->L4.TCPHeader->SrcPort), false);
+				t->Token[11] = PortStr(pl->Cedar, Endian16(p->L4.TCPHeader->DstPort), false);
+
+				ToStr(tmp, Endian32(p->L4.TCPHeader->SeqNumber));
+				t->Token[12] = CopyStr(tmp);
+
+				ToStr(tmp, Endian32(p->L4.TCPHeader->AckNumber));
+				t->Token[13] = CopyStr(tmp);
+
+				snprintf(tmp, sizeof(tmp), "WindowSize=%u", Endian16(p->L4.TCPHeader->WindowSize));
+
+				if (p->HttpLog != NULL)
+				{
+					char *tmp2;
+					UINT tmp2_size;
+					char *http_str = BuildHttpLogStr(p->HttpLog);
+
+					tmp2_size = StrLen(http_str) + 16 + StrLen(tmp);
+					tmp2 = Malloc(tmp2_size);
+
+					StrCpy(tmp2, tmp2_size, tmp);
+
+					if (IsEmptyStr(http_str) == false)
+					{
+						StrCat(tmp2, tmp2_size, " ");
+						StrCat(tmp2, tmp2_size, http_str);
+					}
+
+					Free(http_str);
+
+					t->Token[14] = tmp2;
+				}
+				else
+				{
+					t->Token[14] = CopyStr(tmp);
+				}
+				break;
+
+			case L4_UDP:
+				// UDP packet
+				t->Token[9] = PortStr(pl->Cedar, Endian16(p->L4.UDPHeader->SrcPort), true);
+				t->Token[11] = PortStr(pl->Cedar, Endian16(p->L4.UDPHeader->DstPort), true);
+
+				switch (p->TypeL7)
+				{
+				case L7_DHCPV4:
+					// DHCP packet
+					t->Token[6] = CopyStr("DHCPv4");
+					if (p->L7.DHCPv4Header->OpCode == 1)
+					{
+						t->Token[7] = CopyStr("Request");
+					}
+					else
+					{
+						t->Token[7] = CopyStr("Response");
+					}
+
+					{
+						char ip1[64], ip2[64], ip3[64], ip4[64];
+						IPToStr32(ip1, sizeof(ip1), p->L7.DHCPv4Header->ClientIP);
+						IPToStr32(ip2, sizeof(ip2), p->L7.DHCPv4Header->YourIP);
+						IPToStr32(ip3, sizeof(ip3), p->L7.DHCPv4Header->ServerIP);
+						IPToStr32(ip4, sizeof(ip4), p->L7.DHCPv4Header->RelayIP);
+
+						snprintf(tmp, sizeof(tmp),
+							"TransactionId=%u ClientIP=%s YourIP=%s ServerIP=%s RelayIP=%s",
+							Endian32(p->L7.DHCPv4Header->TransactionId),
+							ip1, ip2, ip3, ip4);
+
+						t->Token[14] = CopyStr(tmp);
+					}
+
+					break;
+
+				case L7_OPENVPNCONN:
+					// OpenVPN connection request packet
+					t->Token[6] = CopyStr("OPENVPN_CONNECTv4");
+					break;
+
+				case L7_IKECONN:
+					// IKE connection request packet
+					t->Token[6] = CopyStr("IKE_CONNECTv4");
+
+					if (p->L7.IkeHeader != NULL)
+					{
+						if (p->L7.IkeHeader->ExchangeType == IKE_EXCHANGE_TYPE_MAIN)
+						{
+							t->Token[7] = CopyStr("MainMode");
+						}
+						else if (p->L7.IkeHeader->ExchangeType == IKE_EXCHANGE_TYPE_MAIN)
+						{
+							t->Token[7] = CopyStr("AgressiveMode");
+						}
+
+						{
+							Format(tmp, sizeof(tmp), "InitiatorCookie=%I64u ResponderCookie=%I64u "
+								"Version=0x%x ExchangeType=0x%x Flag=0x%x MessageId=%u MessageSize=%u",
+								Endian64(p->L7.IkeHeader->InitiatorCookie),
+								Endian64(p->L7.IkeHeader->ResponderCookie),
+								p->L7.IkeHeader->Version,
+								p->L7.IkeHeader->ExchangeType,
+								p->L7.IkeHeader->Flag,
+								Endian32(p->L7.IkeHeader->MessageId),
+								Endian32(p->L7.IkeHeader->MessageSize));
+
+							t->Token[14] = CopyStr(tmp);
+						}
+					}
+					break;
+
+				case L7_DNS:
+					// DNS query
+					t->Token[6] = CopyStr("DNSv4");
+					t->Token[7] = CopyStr("DNS_Query");
+					t->Token[14] = CopyStr(p->DnsQueryHost);
+					break;
+
+				default:
+					// Unknown Packet
+					t->Token[6] = CopyStr("UDPv4");
+					break;
+				}
+				break;
+
+			case L4_FRAGMENT:
+				// Fragment
+				snprintf(tmp, sizeof(tmp), "IPv4_Fragment(0x%02X)", p->L3.IPv4Header->Protocol);
+				t->Token[6] = CopyStr(tmp);
+				break;
+
+			case L4_UNKNOWN:
+				// Unknown Packet
+				snprintf(tmp, sizeof(tmp), "IPv4(0x%02X)", p->L3.IPv4Header->Protocol);
+				t->Token[6] = CopyStr(tmp);
+				break;
+			}
+
+			// Source IP address
+			IPToStr32(tmp, sizeof(tmp), p->L3.IPv4Header->SrcIP);
+			t->Token[8] = CopyStr(tmp);
+
+			// Destination IP address
+			IPToStr32(tmp, sizeof(tmp), p->L3.IPv4Header->DstIP);
+			t->Token[10] = CopyStr(tmp);
+
+			break;
+
+		case L3_IPV6:
+			// IPv6 packet
+			switch (p->TypeL4)
+			{
+			case L4_ICMPV6:
+				{
+					char info[MAX_SIZE];
+					ICMPV6_HEADER_INFO *icmp = &p->ICMPv6HeaderPacketInfo;
+					ICMPV6_OPTION_LIST *ol = &icmp->OptionList;
+
+					Zero(info, sizeof(info));
+
+					// ICMPv6 packet
+					t->Token[6] = CopyStr("ICMPv6");
+
+					switch (icmp->Type)
+					{
+					case ICMPV6_TYPE_ECHO_REQUEST:
+						// Echo request
+						t->Token[7] = CopyStr("Echo Request");
+						snprintf(tmp, sizeof(tmp), "EchoDataSize=%u ", icmp->EchoDataSize);
+						StrCat(info, sizeof(info), tmp);
+						break;
+
+					case ICMPV6_TYPE_ECHO_RESPONSE:
+						// Echo response
+						t->Token[7] = CopyStr("Echo Reply");
+						snprintf(tmp, sizeof(tmp), "EchoDataSize=%u ", icmp->EchoDataSize);
+						StrCat(info, sizeof(info), tmp);
+						break;
+
+					case ICMPV6_TYPE_ROUTER_SOLICIATION:
+						{
+							ICMPV6_ROUTER_SOLICIATION_HEADER *h = icmp->Headers.RouterSoliciationHeader;
+							// Router Solicitation
+							t->Token[7] = CopyStr("Router Soliciation");
+
+							if (h != NULL)
+							{
+								// No additional information
+							}
+						}
+						break;
+
+					case ICMPV6_TYPE_ROUTER_ADVERTISEMENT:
+						{
+							ICMPV6_ROUTER_ADVERTISEMENT_HEADER *h = icmp->Headers.RouterAdvertisementHeader;
+							// Router Advertisement
+							t->Token[7] = CopyStr("Router Advertisement");
+
+							if (h != NULL)
+							{
+								snprintf(tmp, sizeof(tmp), "CurHopLimit=%u "
+									"Flags=0x%02X Lifetime=%u ",
+									h->CurHopLimit, h->Flags, Endian16(h->Lifetime));
+								StrCat(info, sizeof(info), tmp);
+							}
+						}
+						break;
+
+					case ICMPV6_TYPE_NEIGHBOR_SOLICIATION:
+						{
+							ICMPV6_NEIGHBOR_SOLICIATION_HEADER *h = icmp->Headers.NeighborSoliciationHeader;
+							// Neighbor Solicitation
+							t->Token[7] = CopyStr("Neighbor Soliciation");
+
+							if (h != NULL)
+							{
+								char tmp2[MAX_SIZE];
+
+								IP6AddrToStr(tmp2, sizeof(tmp2), &h->TargetAddress);
+
+								snprintf(tmp, sizeof(tmp), "TargetAddress=%s ",
+									tmp2);
+								StrCat(info, sizeof(info), tmp);
+							}
+						}
+						break;
+
+					case ICMPV6_TYPE_NEIGHBOR_ADVERTISEMENT:
+						{
+							ICMPV6_NEIGHBOR_ADVERTISEMENT_HEADER *h = icmp->Headers.NeighborAdvertisementHeader;
+							// Neighbor Advertisement
+							t->Token[7] = CopyStr("Neighbor Advertisement");
+
+							if (h != NULL)
+							{
+								char tmp2[MAX_SIZE];
+
+								IP6AddrToStr(tmp2, sizeof(tmp2), &h->TargetAddress);
+
+								snprintf(tmp, sizeof(tmp), "TargetAddress=%s Flags=0x%02X ",
+									tmp2, h->Flags);
+								StrCat(info, sizeof(info), tmp);
+							}
+						}
+						break;
+
+					default:
+						{
+							snprintf(tmp, sizeof(tmp), "Type=%u", icmp->Type);
+							t->Token[7] = CopyStr(tmp);
+						}
+						break;
+					}
+
+					// Option data
+					if (ol->SourceLinkLayer != NULL)
+					{
+						char tmp2[MAX_SIZE];
+						BinToStr(tmp2, sizeof(tmp2), ol->SourceLinkLayer->Address, 6);
+						snprintf(tmp, sizeof(tmp), "SourceLinkLayer=%s ", tmp2);
+						StrCat(info, sizeof(info), tmp);
+					}
+					if (ol->TargetLinkLayer != NULL)
+					{
+						char tmp2[MAX_SIZE];
+						BinToStr(tmp2, sizeof(tmp2), ol->TargetLinkLayer->Address, 6);
+						snprintf(tmp, sizeof(tmp), "TargetLinkLayer=%s ", tmp2);
+						StrCat(info, sizeof(info), tmp);
+					}
+					if (ol->Prefix != NULL)
+					{
+						char tmp2[MAX_SIZE];
+						IP6AddrToStr(tmp2, sizeof(tmp2), &ol->Prefix->Prefix);
+						snprintf(tmp, sizeof(tmp), "Prefix=%s/%u PrefixFlag=0x%02X ", tmp2,
+							ol->Prefix->SubnetLength, ol->Prefix->Flags);
+						StrCat(info, sizeof(info), tmp);
+					}
+					if (ol->Mtu != NULL)
+					{
+						snprintf(tmp, sizeof(tmp), "Mtu=%u ", Endian32(ol->Mtu->Mtu));
+						StrCat(info, sizeof(info), tmp);
+					}
+
+					Trim(info);
+
+					if (IsEmptyStr(info) == false)
+					{
+						t->Token[14] = CopyStr(info);
+					}
+				}
+				break;
+
+			case L4_TCP:
+				// TCP packet
+				tcp_conn = false;
+				if (p->L4.TCPHeader->Flag & TCP_SYN || p->L4.TCPHeader->Flag & TCP_RST || p->L4.TCPHeader->Flag & TCP_FIN)
+				{
+					tcp_conn = true;
+				}
+				t->Token[6] = CopyStr(tcp_conn ? "TCP_CONNECTv6" : "TCP_DATAv6");
+				t->Token[7] = TcpFlagStr(p->L4.TCPHeader->Flag);
+
+				t->Token[9] = PortStr(pl->Cedar, Endian16(p->L4.TCPHeader->SrcPort), false);
+				t->Token[11] = PortStr(pl->Cedar, Endian16(p->L4.TCPHeader->DstPort), false);
+
+				ToStr(tmp, Endian32(p->L4.TCPHeader->SeqNumber));
+				t->Token[12] = CopyStr(tmp);
+
+				ToStr(tmp, Endian32(p->L4.TCPHeader->AckNumber));
+				t->Token[13] = CopyStr(tmp);
+
+				snprintf(tmp, sizeof(tmp), "WindowSize=%u", Endian16(p->L4.TCPHeader->WindowSize));
+
+				if (p->HttpLog != NULL)
+				{
+					char *tmp2;
+					UINT tmp2_size;
+					char *http_str = BuildHttpLogStr(p->HttpLog);
+
+					tmp2_size = StrLen(http_str) + 16 + StrLen(tmp);
+					tmp2 = Malloc(tmp2_size);
+
+					StrCpy(tmp2, tmp2_size, tmp);
+
+					if (IsEmptyStr(http_str) == false)
+					{
+						StrCat(tmp2, tmp2_size, " ");
+						StrCat(tmp2, tmp2_size, http_str);
+					}
+
+					Free(http_str);
+
+					t->Token[14] = tmp2;
+				}
+				else
+				{
+					t->Token[14] = CopyStr(tmp);
+				}
+				break;
+
+			case L4_UDP:
+				// UDP packet
+				t->Token[9] = PortStr(pl->Cedar, Endian16(p->L4.UDPHeader->SrcPort), true);
+				t->Token[11] = PortStr(pl->Cedar, Endian16(p->L4.UDPHeader->DstPort), true);
+
+				switch (p->TypeL7)
+				{
+				case L7_OPENVPNCONN:
+					// OpenVPN connection request packet
+					t->Token[6] = CopyStr("OPENVPN_CONNECTv6");
+					break;
+
+				case L7_IKECONN:
+					// IKE connection request packet
+					t->Token[6] = CopyStr("IKE_CONNECTv6");
+
+					if (p->L7.IkeHeader != NULL)
+					{
+						if (p->L7.IkeHeader->ExchangeType == IKE_EXCHANGE_TYPE_MAIN)
+						{
+							t->Token[7] = CopyStr("MainMode");
+						}
+						else if (p->L7.IkeHeader->ExchangeType == IKE_EXCHANGE_TYPE_MAIN)
+						{
+							t->Token[7] = CopyStr("AgressiveMode");
+						}
+
+						{
+							Format(tmp, sizeof(tmp), "InitiatorCookie=%I64u ResponderCookie=%I64u "
+								"Version=0x%x ExchangeType=0x%x Flag=0x%x MessageId=%u MessageSize=%u",
+								Endian64(p->L7.IkeHeader->InitiatorCookie),
+								Endian64(p->L7.IkeHeader->ResponderCookie),
+								p->L7.IkeHeader->Version,
+								p->L7.IkeHeader->ExchangeType,
+								p->L7.IkeHeader->Flag,
+								Endian32(p->L7.IkeHeader->MessageId),
+								Endian32(p->L7.IkeHeader->MessageSize));
+
+							t->Token[14] = CopyStr(tmp);
+						}
+					}
+					break;
+
+				case L7_DNS:
+					// DNS query
+					t->Token[6] = CopyStr("DNSv6");
+					t->Token[7] = CopyStr("DNS_Query");
+					t->Token[14] = CopyStr(p->DnsQueryHost);
+					break;
+
+				default:
+					t->Token[6] = CopyStr("UDPv6");
+					break;
+				}
+				break;
+
+			case L4_FRAGMENT:
+				// Fragment packet
+				snprintf(tmp, sizeof(tmp), "IPv6_Fragment(0x%02X)", p->IPv6HeaderPacketInfo.Protocol);
+				t->Token[6] = CopyStr(tmp);
+				break;
+
+			case L4_UNKNOWN:
+				// Unknown Packet
+				snprintf(tmp, sizeof(tmp), "IPv6(0x%02X)", p->IPv6HeaderPacketInfo.Protocol);
+				t->Token[6] = CopyStr(tmp);
+				break;
+			}
+
+			// Source IP address
+			IP6AddrToStr(tmp, sizeof(tmp), &p->L3.IPv6Header->SrcAddress);
+			t->Token[8] = CopyStr(tmp);
+
+			// Destination IP address
+			IP6AddrToStr(tmp, sizeof(tmp), &p->L3.IPv6Header->DestAddress);
+			t->Token[10] = CopyStr(tmp);
+
+			break;
+
+		case L3_UNKNOWN:
+			// Unknown Packet
+			snprintf(tmp, sizeof(tmp), "Proto=0x%04X", Endian16(p->MacHeader->Protocol));
+			t->Token[6] = CopyStr(tmp);
+			break;
+		}
+
+		if (p->PacketData != NULL && (pl->PurePacket == false || pl->PurePacketNoPayload == false))
+		{
+			char *data = Malloc(p->PacketSize * 2 + 1);
+			BinToStr(data, p->PacketSize * 2 + 1, p->PacketData, p->PacketSize);
+			t->Token[15] = data;
+		}
+
+		// Physical IP addresses
+		if (StrLen(pl->SrcPhysicalIP) != 0)
+		{
+			t->Token[16] = CopyStr(pl->SrcPhysicalIP);
+		}
+		if (StrLen(pl->DestPhysicalIP) != 0)
+		{
+			t->Token[17] = CopyStr(pl->DestPhysicalIP);
+		}
+	}
+	else
+	{
+		t->Token[6] = CopyUniToUtf(_UU("LH_PACKET_LOG_NO_LOG_OSS"));
+	}
+
+	s = GenCsvLine(t);
+	FreeToken(t);
+
+	// Discard the packet data
+	if (pl->PurePacket == false)
+	{
+		FreeClonePacket(p);
+	}
+	else
+	{
+		Free(p->PacketData);
+		FreePacket(p);
+	}
+
+	// Release the session
+	if (pl->SrcSession != NULL)
+	{
+		Dec(pl->SrcSession->LoggingRecordCount);
+		ReleaseSession(pl->SrcSession);
+	}
+	Free(pl);
+
+	return s;
 }
 
 // Convert TCP flags to a string
@@ -1574,7 +2502,49 @@ bool MakeLogFileName(LOG *g, char *name, UINT size, char *dir, char *prefix, UIN
 	}
 	else
 	{
-		snprintf(tmp2, sizeof(tmp2), "~%02u", num);
+		UINT64 max_log_size = GetMaxLogSize();
+		if (max_log_size == MAX_LOG_SIZE_DEFAULT)
+		{
+			snprintf(tmp2, sizeof(tmp2), "~%02u", num);
+		}
+		else
+		{
+			char tag[32];
+			char c = '2';
+			if (max_log_size >= 1000000000ULL)
+			{
+				c = '3';
+			}
+			else if (max_log_size >= 100000000ULL)
+			{
+				c = '4';
+			}
+			else if (max_log_size >= 10000000ULL)
+			{
+				c = '5';
+			}
+			else if (max_log_size >= 1000000ULL)
+			{
+				c = '6';
+			}
+			else if (max_log_size >= 100000ULL)
+			{
+				c = '7';
+			}
+			else if (max_log_size >= 10000ULL)
+			{
+				c = '8';
+			}
+			else if (max_log_size >= 1000ULL)
+			{
+				c = '9';
+			}
+
+			StrCpy(tag, sizeof(tag), "~%02u");
+			tag[3] = c;
+
+			snprintf(tmp2, sizeof(tmp2), tag, num);
+		}
 	}
 
 	if (strcmp(old_datestr, tmp) != 0)
@@ -1618,6 +2588,30 @@ void WaitLogFlush(LOG *g)
 	}
 }
 
+// Set the max log size
+void SetMaxLogSize(UINT64 size)
+{
+	if (size == 0)
+	{
+		size = MAX_LOG_SIZE_DEFAULT;
+	}
+
+	logger_max_log_size = size;
+}
+
+// Get the max log size
+UINT64 GetMaxLogSize()
+{
+	UINT64 ret = logger_max_log_size;
+
+	if (ret == 0)
+	{
+		ret = MAX_LOG_SIZE_DEFAULT;
+	}
+
+	return ret;
+}
+
 // Logging thread
 void LogThread(THREAD *thread, void *param)
 {
@@ -1694,7 +2688,7 @@ void LogThread(THREAD *thread, void *param)
 			}
 #endif	// OS_WIN32
 
-			if (b->Size > g->MaxLogFileSize)
+			if (b->Size > GetMaxLogSize())
 			{
 				// Erase if the size of the buffer is larger than the maximum log file size
 				ClearBuf(b);
@@ -1705,7 +2699,7 @@ void LogThread(THREAD *thread, void *param)
 				// Write the contents of the buffer to the file
 				if (io != NULL)
 				{
-					if ((g->CurrentFilePointer + (UINT64)b->Size) > g->MaxLogFileSize)
+					if ((g->CurrentFilePointer + (UINT64)b->Size) > GetMaxLogSize())
 					{
 						if (g->log_number_incremented == false)
 						{
@@ -1739,7 +2733,7 @@ void LogThread(THREAD *thread, void *param)
 					// Write the contents of the buffer to the file
 					if (io != NULL)
 					{
-						if ((g->CurrentFilePointer + (UINT64)b->Size) > g->MaxLogFileSize)
+						if ((g->CurrentFilePointer + (UINT64)b->Size) > GetMaxLogSize())
 						{
 							if (g->log_number_incremented == false)
 							{
@@ -1811,7 +2805,7 @@ void LogThread(THREAD *thread, void *param)
 					{
 						if (log_date_changed)
 						{
-							if ((g->CurrentFilePointer + (UINT64)b->Size) <= g->MaxLogFileSize)
+							if ((g->CurrentFilePointer + (UINT64)b->Size) <= GetMaxLogSize())
 							{
 								if (FileWrite(io, b->Buf, b->Size) == false)
 								{
@@ -2037,7 +3031,6 @@ LOG *NewLog(char *dir, char *prefix, UINT switch_type)
 	g->SwitchType = switch_type;
 	g->RecordQueue = NewQueue();
 	g->Event = NewEvent();
-	g->MaxLogFileSize = MAX_LOG_SIZE;
 	g->FlushEvent = NewEvent();
 
 	g->Thread = NewThread(LogThread, g);
@@ -2048,7 +3041,3 @@ LOG *NewLog(char *dir, char *prefix, UINT switch_type)
 }
 
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
