@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Cedar Communication Module
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // NM.c
@@ -116,6 +146,87 @@
 // Global variable
 static NM *nm = NULL;
 
+// Dialog proc for the push routing option
+UINT NmEditPushRouteProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam, void *param)
+{
+	SM_HUB *r = (SM_HUB *)param;
+	char *str = NULL;
+	// Validate arguments
+	if (hWnd == NULL)
+	{
+		return 0;
+	}
+
+	switch (msg)
+	{
+	case WM_INITDIALOG:
+		SetTextA(hWnd, E_TEXT, r->CurrentPushRouteStr);
+		Focus(hWnd, E_TEXT);
+
+		SetIcon(hWnd, 0, ICO_PROTOCOL);
+		break;
+
+	case WM_COMMAND:
+		switch (wParam)
+		{
+		case IDOK:
+			str = GetTextA(hWnd, E_TEXT);
+			if (str != NULL)
+			{
+				bool ok = true;
+
+				if (CheckClasslessRouteTableStr(str) == false)
+				{
+					if (MsgBox(hWnd, MB_ICONWARNING | MB_OKCANCEL | MB_DEFBUTTON2, _UU("NM_PUSH_ROUTE_WARNING")) == IDCANCEL)
+					{
+						ok = false;
+					}
+				}
+
+				if (ok)
+				{
+					if (IsEmptyStr(str) == false)
+					{
+						if (GetCapsBool(r->p->CapsList, "b_suppport_push_route") == false)
+						{
+							MsgBox(hWnd, MB_ICONEXCLAMATION, _UU("ERR_147"));
+						}
+					}
+
+					StrCpy(r->CurrentPushRouteStr, sizeof(r->CurrentPushRouteStr), str);
+
+					EndDialog(hWnd, 1);
+				}
+
+				Free(str);
+			}
+			break;
+
+		case IDCANCEL:
+			Close(hWnd);
+			break;
+		}
+		break;
+
+	case WM_CLOSE:
+		EndDialog(hWnd, 0);
+		break;
+	}
+
+	return 0;
+}
+
+// Edit dialog for the push routing option
+bool NmEditPushRoute(HWND hWnd, SM_HUB *r)
+{
+	// Validate arguments
+	if (r == NULL)
+	{
+		return false;
+	}
+
+	return Dialog(hWnd, D_NM_PUSH, NmEditPushRouteProc, r);
+}
 
 // Change Password dialog
 UINT NmChangePasswordProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam, void *param)
@@ -673,6 +784,7 @@ bool NmStatus(HWND hWnd, SM_SERVER *s, void *param)
 	LvInsertAdd(b, ICO_PROTOCOL_DHCP, NULL, 2, _UU("NM_STATUS_DHCP"), tmp);
 
 	LvInsertAdd(b, ICO_MACHINE, NULL, 2, _UU("SM_SNAT_IS_KERNEL"), t.IsKernelMode ? _UU("SEC_YES") : _UU("SEC_NO"));
+	LvInsertAdd(b, ICO_MACHINE, NULL, 2, _UU("SM_SNAT_IS_RAW"), t.IsRawIpMode ? _UU("SEC_YES") : _UU("SEC_NO"));
 
 	LvInsertEnd(b, hWnd, L_STATUS);
 
@@ -736,6 +848,8 @@ void NmEditVhOptionInit(HWND hWnd, SM_HUB *r)
 		return;
 	}
 
+	SetIcon(hWnd, 0, ICO_ROUTER);
+
 	FormatText(hWnd, S_TITLE, r->HubName);
 
 	Zero(&t, sizeof(VH_OPTION));
@@ -786,6 +900,15 @@ void NmEditVhOptionInit(HWND hWnd, SM_HUB *r)
 	SetTextA(hWnd, E_DOMAIN, t.DhcpDomainName);
 	Check(hWnd, R_SAVE_LOG, t.SaveLog);
 
+	StrCpy(r->CurrentPushRouteStr, sizeof(r->CurrentPushRouteStr), t.DhcpPushRoutes);
+
+	if (GetCapsBool(r->p->CapsList, "b_suppport_push_route_config") == false)
+	{
+		Disable(hWnd, S_1);
+		Disable(hWnd, S_2);
+		Disable(hWnd, B_PUSH);
+	}
+
 	NmEditVhOptionUpdate(hWnd, r);
 
 }
@@ -920,6 +1043,9 @@ void NmEditVhOptionOnOk(HWND hWnd, SM_HUB *r)
 	NmEditVhOptionFormToVH(hWnd, &t);
 	StrCpy(t.HubName, sizeof(t.HubName), r->HubName);
 
+	t.ApplyDhcpPushRoutes = true;
+	StrCpy(t.DhcpPushRoutes, sizeof(t.DhcpPushRoutes), r->CurrentPushRouteStr);
+
 	if (CALL(hWnd, ScSetSecureNATOption(r->Rpc, &t)))
 	{
 		EndDialog(hWnd, true);
@@ -987,6 +1113,10 @@ UINT NmEditVhOptionProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam, void
 				Focus(hWnd, E_DHCP_START);
 			}
 			break;
+
+		case B_PUSH:
+			NmEditPushRoute(hWnd, r);
+			break;
 		}
 
 		break;
@@ -1004,6 +1134,7 @@ void NmEditVhOption(HWND hWnd, SM_HUB *r)
 		return;
 	}
 
+	Zero(r->CurrentPushRouteStr, sizeof(r->CurrentPushRouteStr));
 	Dialog(hWnd, D_NM_OPTION, NmEditVhOptionProc, r);
 }
 
@@ -1510,7 +1641,3 @@ void NMExec()
 
 #endif
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
