@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Cedar Communication Module
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Server.h
@@ -99,7 +129,7 @@
 #define	SERVER_DEF_PORTS_INCLIENT_DYN_MAX	1999
 
 extern char *SERVER_CONFIG_FILE_NAME;
-#define	SERVER_DEFAULT_CIPHER_NAME		"RC4-MD5"
+#define	SERVER_DEFAULT_CIPHER_NAME		"AES128-SHA"
 #define	SERVER_DEFAULT_CERT_DAYS		(365 * 10)
 #define	SERVER_DEFAULT_HUB_NAME			"DEFAULT"
 #define	SERVER_DEFAULT_BRIDGE_NAME		"BRIDGE"
@@ -264,6 +294,7 @@ struct SERVER
 	bool DisableIntelAesAcceleration;	// Disable the Intel AES acceleration
 	bool NoMoreSave;					// Do not save any more
 	bool EnableConditionalAccept;		// Apply the Conditional Accept the Listener
+	bool EnableLegacySSL;				// Enable Legacy SSL
 
 	volatile bool Halt;					// Halting flag
 	LOCK *lock;							// Lock
@@ -321,6 +352,9 @@ struct SERVER
 	AZURE_CLIENT *AzureClient;			// VPN Azure client
 	bool EnableVpnAzure;				// Flag whether VPN Azure client is enabled
 
+	bool DisableGetHostNameWhenAcceptTcp;	// Disable GetHostName when accepting TCP
+	bool DisableCoreDumpOnUnix;			// Disable core dump on UNIX
+
 	TINY_LOG *DebugLog;					// Debug log
 
 	DYNAMIC_LISTENER *DynListenerIcmp;	// VPN over ICMP listener
@@ -333,6 +367,8 @@ struct SERVER
 
 
 	volatile UINT NatTGlobalUdpPort;	// NAT-T global UDP port
+
+	bool StrictSyslogDatetimeFormat;	// Make syslog datetime format strict RFC3164
 };
 
 
@@ -388,6 +424,47 @@ struct LOG_FILE
 };
 
 
+// Global server flags
+#define	NUM_GLOBAL_SERVER_FLAGS			128
+#define	GSF_DISABLE_PUSH_ROUTE			1
+#define	GSF_DISABLE_RADIUS_AUTH			2
+#define	GSF_DISABLE_CERT_AUTH			3
+#define	GSF_DISABLE_DEEP_LOGGING		4
+#define	GSF_DISABLE_AC					5
+#define	GSF_DISABLE_SYSLOG				6
+#define	GSF_SHOW_OSS_MSG				7
+#define	GSF_LOCALBRIDGE_NO_DISABLE_OFFLOAD	8
+#define	GSF_DISABLE_SESSION_RECONNECT	9
+
+// Global parameters
+#define	NUM_GLOBAL_PARAMS					128
+#define	GP_MAX_SEND_SOCKET_QUEUE_SIZE		1
+#define	GP_MIN_SEND_SOCKET_QUEUE_SIZE		2
+#define	GP_MAX_SEND_SOCKET_QUEUE_NUM		3
+#define	GP_SELECT_TIME						4
+#define	GP_SELECT_TIME_FOR_NAT				5
+#define	GP_MAX_STORED_QUEUE_NUM				6
+#define	GP_MAX_BUFFERING_PACKET_SIZE		7
+#define	GP_HUB_ARP_SEND_INTERVAL			8
+#define	GP_MAC_TABLE_EXPIRE_TIME			9
+#define	GP_IP_TABLE_EXPIRE_TIME				10
+#define	GP_IP_TABLE_EXPIRE_TIME_DHCP		11
+#define	GP_STORM_CHECK_SPAN					12
+#define	GP_STORM_DISCARD_VALUE_START		13
+#define	GP_STORM_DISCARD_VALUE_END			14
+#define	GP_MAX_MAC_TABLES					15
+#define	GP_MAX_IP_TABLES					16
+#define	GP_MAX_HUB_LINKS					17
+#define	GP_MEM_FIFO_REALLOC_MEM_SIZE		18
+#define	GP_QUEUE_BUDGET						19
+#define	GP_FIFO_BUDGET						20
+
+extern UINT vpn_global_parameters[NUM_GLOBAL_PARAMS];
+
+#define	VPN_GP(id, default_value)	((UINT)(vpn_global_parameters[(id)] != 0 ? vpn_global_parameters[(id)] : (default_value)))
+
+
+
 // Virtual HUB creation history
 struct SERVER_HUB_CREATE_HISTORY
 {
@@ -397,7 +474,7 @@ struct SERVER_HUB_CREATE_HISTORY
 
 // Function prototype declaration
 SERVER *SiNewServer(bool bridge);
-SERVER *SiNewServerEx(bool bridge, bool in_client_inner_server);
+SERVER *SiNewServerEx(bool bridge, bool in_client_inner_server, bool relay_server);
 void SiReleaseServer(SERVER *s);
 void SiCleanupServer(SERVER *s);
 void StStartServer(bool bridge);
@@ -439,6 +516,9 @@ void SiWriteListenerCfg(FOLDER *f, SERVER_LISTENER *r);
 void SiLoadListenerCfg(SERVER *s, FOLDER *f);
 void SiWriteServerCfg(FOLDER *f, SERVER *s);
 void SiLoadServerCfg(SERVER *s, FOLDER *f);
+void SiWriteGlobalParamsCfg(FOLDER *f);
+void SiLoadGlobalParamsCfg(FOLDER *f);
+void SiLoadGlobalParamItem(UINT id, UINT value);
 void SiWriteTraffic(FOLDER *parent, char *name, TRAFFIC *t);
 void SiWriteTrafficInner(FOLDER *parent, char *name, TRAFFIC_ENTRY *e);
 void SiLoadTrafficInner(FOLDER *parent, char *name, TRAFFIC_ENTRY *e);
@@ -599,6 +679,8 @@ UINT SiDebugProcGetIPsecMessageDisplayedValue(SERVER *s, char *in_str, char *ret
 UINT SiDebugProcSetIPsecMessageDisplayedValue(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
 UINT SiDebugProcGetVgsMessageDisplayedValue(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
 UINT SiDebugProcSetVgsMessageDisplayedValue(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
+UINT SiDebugProcGetCurrentTcpSendQueueLength(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
+UINT SiDebugProcGetCurrentGetIPThreadCount(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
 
 typedef UINT (SI_DEBUG_PROC)(SERVER *s, char *in_str, char *ret_str, UINT ret_str_size);
 
@@ -621,6 +703,7 @@ void InRpcSysLogSetting(SYSLOG_SETTING *t, PACK *p);
 void OutRpcSysLogSetting(PACK *p, SYSLOG_SETTING *t);
 
 void GetServerCaps(SERVER *s, CAPSLIST *t);
+void FlushServerCaps(SERVER *s);
 bool GetServerCapsBool(SERVER *s, char *name);
 UINT GetServerCapsInt(SERVER *s, char *name);
 void GetServerCapsMain(SERVER *s, CAPSLIST *t);
@@ -628,6 +711,10 @@ void InitServerCapsCache(SERVER *s);
 void FreeServerCapsCache(SERVER *s);
 void DestroyServerCapsCache(SERVER *s);
 
+void SetGlobalServerFlag(UINT index, UINT value);
+UINT GetGlobalServerFlag(UINT index);
+void UpdateGlobalServerFlags(SERVER *s, CAPSLIST *t);
+
 
 bool IsAdminPackSupportedServerProduct(char *name);
 
@@ -654,11 +741,12 @@ void SiApplyAzureConfig(SERVER *s, DDNS_CLIENT_STATUS *ddns_status);
 void SiSetAzureEnable(SERVER *s, bool enabled);
 bool SiGetAzureEnable(SERVER *s);
 
+void SiUpdateCurrentRegion(CEDAR *c, char *region, bool force_update);
+void SiGetCurrentRegion(CEDAR *c, char *region, UINT region_size);
+bool SiIsEnterpriseFunctionsRestrictedOnOpenSource(CEDAR *c);
+bool SiCheckCurrentRegion(CEDAR *c, char *r);
+
 #endif	// SERVER_H
 
 
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
