@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Cedar Communication Module
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Session.c
@@ -107,11 +137,14 @@ void SessionMain(SESSION *s)
 	TRAFFIC t;
 	SOCK *msgdlg_sock = NULL;
 	SOCK *nicinfo_sock = NULL;
+	bool is_server_session = false;
+	bool lock_receive_blocks_queue = false;
 	// Validate arguments
 	if (s == NULL)
 	{
 		return;
 	}
+
 	Debug("SessionMain: %s\n", s->Name);
 
 	Notify(s, CLIENT_NOTIFY_ACCOUNT_CHANGED);
@@ -129,6 +162,19 @@ void SessionMain(SESSION *s)
 	policy = s->Policy;
 
 	// Initialize the packet adapter
+#ifdef	OS_WIN32
+	if (s->IsVPNClientAndVLAN_Win32)
+	{
+		MsBeginVLanCard();
+
+		if (MsIsVLanCardShouldStop())
+		{
+			err = ERR_SUSPENDING;
+			goto CLEANUP;
+		}
+	}
+#endif	// OS_WIN32
+
 	pa = s->PacketAdapter;
 	if (pa->Init(s) == false)
 	{
@@ -241,9 +287,14 @@ void SessionMain(SESSION *s)
 		}
 	}
 
+	is_server_session = s->ServerMode;
+
+	lock_receive_blocks_queue = s->LinkModeServer;
+
+	now = Tick64();
+
 	while (true)
 	{
-		now = Tick64();
 		Zero(&t, sizeof(t));
 
 
@@ -268,6 +319,16 @@ void SessionMain(SESSION *s)
 		}
 
 
+		if (is_server_session && s->LinkModeServer == false && s->SecureNATMode == false && s->BridgeMode == false && s->L3SwitchMode == false)
+		{
+			if (s->Hub != NULL && s->Hub->ForceDisableComm)
+			{
+				// Disconnect the session forcibly because the ForceDisableComm flag is set
+				err = ERR_SERVER_CANT_ACCEPT;
+				pa_fail = true;
+			}
+		}
+
 		if (s->InProcMode)
 		{
 			if (c->TubeSock == NULL || IsTubeConnected(c->TubeSock->SendTube) == false || IsTubeConnected(c->TubeSock->RecvTube) == false)
@@ -287,13 +348,22 @@ void SessionMain(SESSION *s)
 				pa_fail = true;
 			}
 		}
-
+		
 		// Chance of additional connection
-		ClientAdditionalConnectChance(s);
+		if (is_server_session == false)
+		{
+			if (GetGlobalServerFlag(GSF_DISABLE_SESSION_RECONNECT) == false)
+			{
+				ClientAdditionalConnectChance(s);
+			}
+		}
 
 		// Receive a block
 		ConnectionReceive(c, s->Cancel1, s->Cancel2);
 
+		// Get the current time
+		now = Tick64();
+
 		if (s->UseUdpAcceleration && s->UdpAccel != NULL && s->UdpAccel->FatalError)
 		{
 			// A serious error occurs during sending any data on UDP socket
@@ -302,8 +372,23 @@ void SessionMain(SESSION *s)
 			pa_fail = true;
 		}
 
+#ifdef	OS_WIN32
+		if (s->IsVPNClientAndVLAN_Win32)
+		{
+			if (MsIsVLanCardShouldStop())
+			{
+				// System is suspending
+				err = ERR_SUSPENDING;
+				pa_fail = true;
+			}
+		}
+#endif	// OS_WIN32
+
 		// Pass the received block to the PacketAdapter
-		LockQueue(c->ReceivedBlocks);
+		if (lock_receive_blocks_queue)
+		{
+			LockQueue(c->ReceivedBlocks);
+		}
 		{
 			BLOCK *b;
 			packet_put = false;
@@ -319,17 +404,33 @@ void SessionMain(SESSION *s)
 
 				update_hub_last_comm = true;
 
-				if (s->ServerMode == false && b->Size >= 14)
+				if (b->Size >= 14)
 				{
-					if (b->Buf[0] & 0x40)
+					if (b->Buf[0] & 0x01)
 					{
-						t.Recv.BroadcastCount++;
-						t.Recv.BroadcastBytes += (UINT64)b->Size;
+						if (is_server_session == false)
+						{
+							t.Recv.BroadcastCount++;
+							t.Recv.BroadcastBytes += (UINT64)b->Size;
+						}
+						else
+						{
+							t.Send.BroadcastCount++;
+							t.Send.BroadcastBytes += (UINT64)b->Size;
+						}
 					}
 					else
 					{
-						t.Recv.UnicastCount++;
-						t.Recv.UnicastBytes += (UINT64)b->Size;
+						if (is_server_session == false)
+						{
+							t.Recv.UnicastCount++;
+							t.Recv.UnicastBytes += (UINT64)b->Size;
+						}
+						else
+						{
+							t.Send.UnicastCount++;
+							t.Send.UnicastBytes += (UINT64)b->Size;
+						}
 					}
 				}
 
@@ -345,7 +446,7 @@ void SessionMain(SESSION *s)
 				Free(b);
 			}
 
-			if (packet_put || s->ServerMode)
+			if (true /* packet_put || is_server_session 2014.7.23 for optimizing */)
 			{
 				PROBE_DATA2("pa->PutPacket", NULL, 0);
 				if (pa->PutPacket(s, NULL, 0) == false)
@@ -356,10 +457,12 @@ void SessionMain(SESSION *s)
 				}
 			}
 		}
-		UnlockQueue(c->ReceivedBlocks);
+		if (lock_receive_blocks_queue)
+		{
+			UnlockQueue(c->ReceivedBlocks);
+		}
 
 		// Add the packet to be transmitted to SendBlocks by acquiring from PacketAdapter
-		LockQueue(c->SendBlocks);
 		{
 			UINT i, max_num = MAX_SEND_SOCKET_QUEUE_NUM;
 			i = 0;
@@ -386,35 +489,75 @@ void SessionMain(SESSION *s)
 				else
 				{
 					bool priority;
+					QUEUE *q = NULL;
 					// Buffering
-					if (s->ServerMode == false && packet_size >= 14)
+					if (packet_size >= 14)
 					{
 						UCHAR *buf = (UCHAR *)packet;
 						if (buf[0] & 0x01)
 						{
-							t.Send.BroadcastCount++;
-							t.Send.BroadcastBytes += (UINT64)packet_size;
+							if (is_server_session == false)
+							{
+								t.Send.BroadcastCount++;
+								t.Send.BroadcastBytes += (UINT64)packet_size;
+							}
+							else
+							{
+								t.Recv.BroadcastCount++;
+								t.Recv.BroadcastBytes += (UINT64)packet_size;
+							}
 						}
 						else
 						{
-							t.Send.UnicastCount++;
-							t.Send.UnicastBytes += (UINT64)packet_size;
+							if (is_server_session == false)
+							{
+								t.Send.UnicastCount++;
+								t.Send.UnicastBytes += (UINT64)packet_size;
+							}
+							else
+							{
+								t.Recv.UnicastCount++;
+								t.Recv.UnicastBytes += (UINT64)packet_size;
+							}
 						}
 					}
 					priority = IsPriorityHighestPacketForQoS(packet, packet_size);
+
 					b = NewBlock(packet, packet_size, s->UseCompress ? 1 : 0);
 					b->PriorityQoS = priority;
-					c->CurrentSendQueueSize += b->Size;
 
 					if (b->PriorityQoS && c->Protocol == CONNECTION_TCP && s->QoS)
 					{
-						InsertQueue(c->SendBlocks2, b);
+						q = c->SendBlocks2;
 					}
 					else
 					{
-						InsertQueue(c->SendBlocks, b);
+						q = c->SendBlocks;
+					}
+
+					if (q->num_item > MAX_STORED_QUEUE_NUM)
+					{
+						q = NULL;
+					}
+
+					if (q != NULL)
+					{
+						c->CurrentSendQueueSize += b->Size;
+						InsertQueue(q, b);
+					}
+					else
+					{
+						FreeBlock(b);
 					}
 				}
+
+				if ((i % 16) == 0)
+				{
+					int diff = ((int)c->CurrentSendQueueSize) - ((int)c->LastPacketQueueSize);
+					CedarAddCurrentTcpQueueSize(c->Cedar, diff);
+					c->LastPacketQueueSize = c->CurrentSendQueueSize;
+				}
+
 				i++;
 				if (i >= max_num)
 				{
@@ -422,15 +565,23 @@ void SessionMain(SESSION *s)
 				}
 			}
 		}
-		UnlockQueue(c->SendBlocks);
 
 		AddTrafficForSession(s, &t);
 
+		if (true)
+		{
+			int diff = ((int)c->CurrentSendQueueSize) - ((int)c->LastPacketQueueSize);
+			CedarAddCurrentTcpQueueSize(c->Cedar, diff);
+			c->LastPacketQueueSize = c->CurrentSendQueueSize;
+		}
+
+		now = Tick64();
+
 		// Send a block
-		ConnectionSend(c);
+		ConnectionSend(c, now);
 
 		// Determine the automatic disconnection
-		if (auto_disconnect_tick != 0 && auto_disconnect_tick <= Tick64())
+		if (auto_disconnect_tick != 0 && auto_disconnect_tick <= now)
 		{
 			err = ERR_AUTO_DISCONNECTED;
 			s->CurrentRetryCount = INFINITE;
@@ -447,9 +598,6 @@ void SessionMain(SESSION *s)
 			break;
 		}
 
-		// Get the current time
-		now = Tick64();
-
 		// Increments the number of logins for user object and Virtual HUB object.
 		// (It's incremented only if the time 30 seconds passed after connection.
 		// If not do this, it will be incremented on DoS attacks or any error.)
@@ -468,7 +616,7 @@ void SessionMain(SESSION *s)
 			}
 		}
 
-		if (s->ServerMode)
+		if (is_server_session)
 		{
 			HUB *hub;
 
@@ -483,15 +631,11 @@ void SessionMain(SESSION *s)
 
 			if (hub != NULL)
 			{
-				Lock(hub->lock);
+				if ((hub->LastIncrementTraffic + INCREMENT_TRAFFIC_INTERVAL) <= now)
 				{
-					if ((hub->LastIncrementTraffic + INCREMENT_TRAFFIC_INTERVAL) <= now)
-					{
-						IncrementHubTraffic(s->Hub);
-						hub->LastIncrementTraffic = now;
-					}
+					hub->LastIncrementTraffic = now;
+					IncrementHubTraffic(s->Hub);
 				}
-				Unlock(hub->lock);
 			}
 		}
 
@@ -506,7 +650,22 @@ void SessionMain(SESSION *s)
 				WHERE;
 			}
 
-			if (s->ServerMode == false && s->ClientOption != NULL && s->ClientOption->ConnectionDisconnectSpan == 0)
+			if (c->Protocol == CONNECTION_TCP)
+			{
+				if (GetGlobalServerFlag(GSF_DISABLE_SESSION_RECONNECT))
+				{
+					UINT num_tcp_connections = Count(c->CurrentNumConnection);
+
+					if (num_tcp_connections == 0)
+					{
+						// All TCP connections are disconnected.
+						// Terminate the session immediately.
+						timeouted = true;
+					}
+				}
+			}
+
+			if (is_server_session == false && s->ClientOption != NULL && s->ClientOption->ConnectionDisconnectSpan == 0)
 			{
 				if (LIST_NUM(s->Connection->Tcp->TcpSockList) < s->MaxConnection)
 				{
@@ -558,7 +717,14 @@ CLEANUP:
 
 	if (s->Connection)
 	{
+		int diff =  -((int)s->Connection->LastTcpQueueSize);
+		s->Connection->LastTcpQueueSize = 0;
 		s->Connection->Halt = true;
+		CedarAddCurrentTcpQueueSize(s->Cedar, diff);
+
+		diff = ((int)c->CurrentSendQueueSize) - ((int)c->LastPacketQueueSize);
+		CedarAddCurrentTcpQueueSize(c->Cedar, diff);
+		c->LastPacketQueueSize = c->CurrentSendQueueSize;
 	}
 
 	// Release the packet adapter
@@ -567,6 +733,13 @@ CLEANUP:
 		pa->Free(s);
 	}
 
+#ifdef	OS_WIN32
+	if (s->IsVPNClientAndVLAN_Win32)
+	{
+		MsEndVLanCard();
+	}
+#endif	// OS_WIN32
+
 	if (s->ServerMode == false)
 	{
 		// Cancel to make all additional connection
@@ -1278,20 +1451,26 @@ void ClientThread(THREAD *t, void *param)
 	bool no_save_password = false;
 	bool is_vpngate_connection = false;
 	CEDAR *cedar;
+	bool num_active_sessions_incremented = false;
 	// Validate arguments
 	if (t == NULL || param == NULL)
 	{
 		return;
 	}
 
-	CiIncrementNumActiveSessions();
-
 	Debug("ClientThread 0x%x Started.\n", t);
 
 	s = (SESSION *)param;
 	AddRef(s->ref);
 	s->Thread = t;
 	AddRef(t->ref);
+
+	if (s->LinkModeClient == false)
+	{
+		CiIncrementNumActiveSessions();
+		num_active_sessions_incremented = true;
+	}
+
 	NoticeThreadInit(t);
 
 	cedar = s->Cedar;
@@ -1313,6 +1492,14 @@ void ClientThread(THREAD *t, void *param)
 
 	while (true)
 	{
+		Zero(&s->ServerIP_CacheForNextConnect, sizeof(IP));
+
+		if (s->Link != NULL && ((*s->Link->StopAllLinkFlag) || s->Link->Halting))
+		{
+			s->Err = ERR_USER_CANCEL;
+			break;
+		}
+
 		CLog(s->Cedar->Client, "LC_CONNECT_1", s->ClientOption->AccountName, s->CurrentRetryCount + 1);
 		if (s->LinkModeClient && s->Link != NULL)
 		{
@@ -1394,6 +1581,21 @@ void ClientThread(THREAD *t, void *param)
 		{
 			UINT retry_interval = s->RetryInterval;
 
+			if (s->LinkModeClient)
+			{
+				UINT current_num_links = Count(s->Cedar->CurrentActiveLinks);
+				UINT max_retry_interval = MAX(1000 * current_num_links, retry_interval);
+
+				retry_interval += retry_interval * MIN(s->CurrentRetryCount, 1000);
+				retry_interval = MIN(retry_interval, max_retry_interval);
+
+				// On the cascade client, adjust the retry_interval. (+/- 20%)
+				if (retry_interval >= 1000 && retry_interval <= (60 * 60 * 1000))
+				{
+					retry_interval = (retry_interval * 8 / 10) + (Rand32() % (retry_interval * 4 / 10));
+				}
+			}
+
 			if (s->Err == ERR_HUB_IS_BUSY || s->Err == ERR_LICENSE_ERROR ||
 				s->Err == ERR_HUB_STOPPING || s->Err == ERR_TOO_MANY_USER_SESSION)
 			{
@@ -1641,7 +1843,10 @@ SKIP:
 
 	ReleaseSession(s);
 
-	CiDecrementNumActiveSessions();
+	if (num_active_sessions_incremented)
+	{
+		CiDecrementNumActiveSessions();
+	}
 }
 
 // Name comparison of sessions
@@ -1782,6 +1987,13 @@ SESSION *NewClientSessionEx(CEDAR *cedar, CLIENT_OPTION *option, CLIENT_AUTH *au
 	s->ClientOption = Malloc(sizeof(CLIENT_OPTION));
 	Copy(s->ClientOption, option, sizeof(CLIENT_OPTION));
 
+	if (GetGlobalServerFlag(GSF_DISABLE_SESSION_RECONNECT))
+	{
+		s->ClientOption->DisableQoS = true;
+		s->ClientOption->MaxConnection = 1;
+		s->ClientOption->HalfConnection = false;
+	}
+
 	s->MaxConnection = option->MaxConnection;
 	s->UseEncrypt = option->UseEncrypt;
 	s->UseCompress = option->UseCompress;
@@ -1795,11 +2007,17 @@ SESSION *NewClientSessionEx(CEDAR *cedar, CLIENT_OPTION *option, CLIENT_AUTH *au
 
 	// Hold whether the virtual LAN card is used in client mode
 	s->ClientModeAndUseVLan = (StrLen(s->ClientOption->DeviceName) == 0) ? false : true;
+
 	if (s->ClientOption->NoRoutingTracking)
 	{
 		s->ClientModeAndUseVLan = false;
 	}
 
+	if (pa->Id == PACKET_ADAPTER_ID_VLAN_WIN32)
+	{
+		s->IsVPNClientAndVLAN_Win32 = true;
+	}
+
 	if (StrLen(option->DeviceName) == 0)
 	{
 		// NAT mode
@@ -2050,6 +2268,19 @@ SESSION *NewServerSessionEx(CEDAR *cedar, CONNECTION *c, HUB *h, char *username,
 		{
 			Format(name, sizeof(name), "SID-%s-[%s]-%u", user_name_upper, c->InProcPrefix, Inc(h->SessionCounter));
 		}
+
+		if (h->IsVgsHub || h->IsVgsSuperRelayHub)
+		{
+			UCHAR rand[5];
+			char tmp[32];
+
+			Rand(rand, sizeof(rand));
+
+			BinToStr(tmp, sizeof(tmp), rand, sizeof(rand));
+
+			StrCat(name, sizeof(name), "-");
+			StrCat(name, sizeof(name), tmp);
+		}
 	}
 	else
 	{
@@ -2127,6 +2358,23 @@ SESSION *NewServerSessionEx(CEDAR *cedar, CONNECTION *c, HUB *h, char *username,
 	return s;
 }
 
+// Check whether the specified MAC address is IPC address
+bool IsIpcMacAddress(UCHAR *mac)
+{
+	// Validate arguments
+	if (mac == NULL)
+	{
+		return false;
+	}
+
+	if (mac[0] == 0xCA)
+	{
+		return true;
+	}
+
+	return false;
+}
+
 // Display the session key for debugging
 void DebugPrintSessionKey(UCHAR *session_key)
 {
@@ -2236,7 +2484,3 @@ void Notify(SESSION *s, UINT code)
 }
 
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
