@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Cedar Communication Module
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Win32Com.c
@@ -209,6 +239,134 @@ bool Win32UPnPAddPort(UINT outside_port, UINT inside_port, bool udp, char *local
 	return ret;
 }
 
+// Install the NDIS protocol driver
+bool UninstallNdisProtocolDriver(wchar_t *id, UINT lock_timeout)
+{
+	bool ret = false;
+	HRESULT hr;
+	INetCfg *pNetCfg;
+	// Validate arguments
+	if (id == NULL)
+	{
+		return false;
+	}
+
+	hr = CoCreateInstance(CLSID_CNetCfg, NULL, CLSCTX_INPROC_SERVER, IID_INetCfg, (void **)&pNetCfg);
+
+	if (SUCCEEDED(hr))
+	{
+		INetCfgLock *pLock;
+
+		hr = pNetCfg->QueryInterface(IID_INetCfgLock, (PVOID*)&pLock);
+
+		if (SUCCEEDED(hr))
+		{
+			LPWSTR locked_by;
+
+			hr = pLock->AcquireWriteLock(lock_timeout, L"SoftEther VPN", &locked_by);
+
+			if (SUCCEEDED(hr))
+			{
+				hr = pNetCfg->Initialize(NULL);
+
+				if (SUCCEEDED(hr))
+				{
+					INetCfgComponent *pncc = NULL;
+
+					hr = pNetCfg->FindComponent(id, &pncc);
+
+					if (pncc == NULL || hr == S_FALSE)
+					{
+						hr = E_FAIL;
+					}
+
+					if (SUCCEEDED(hr))
+					{
+						INetCfgClass *pncClass;
+
+						hr = pNetCfg->QueryNetCfgClass(&GUID_DEVCLASS_NETTRANS, IID_INetCfgClass, (void **)&pncClass);
+						if (SUCCEEDED(hr))
+						{
+							INetCfgClassSetup *pncClassSetup;
+
+							hr = pncClass->QueryInterface(IID_INetCfgClassSetup, (void **)&pncClassSetup);
+							if (SUCCEEDED(hr))
+							{
+								OBO_TOKEN obo;
+								wchar_t *c = NULL;
+
+								Zero(&obo, sizeof(obo));
+
+								obo.Type = OBO_USER;
+
+								hr = pncClassSetup->DeInstall(pncc, &obo, &c);
+
+								if (SUCCEEDED(hr))
+								{
+									hr = pNetCfg->Apply();
+
+									if (SUCCEEDED(hr))
+									{
+										ret = true;
+									}
+									else
+									{
+										WHERE;
+										Debug("0x%x\n", hr);
+									}
+								}
+								else
+								{
+									WHERE;
+									Debug("0x%x\n", hr);
+								}
+
+								pncClassSetup->Release();
+							}
+							else
+							{
+								WHERE;
+							}
+
+							pncClass->Release();
+						}
+						else
+						{
+							WHERE;
+						}
+
+						pncc->Release();
+					}
+					else
+					{
+						WHERE;
+					}
+				}
+				else
+				{
+					WHERE;
+				}
+
+				pLock->ReleaseWriteLock();
+			}
+			else
+			{
+				WHERE;
+			}
+
+			pLock->Release();
+		}
+
+		pNetCfg->Release();
+	}
+	else
+	{
+		WHERE;
+	}
+
+	return ret;
+}
+
 // Install the NDIS protocol driver
 bool InstallNdisProtocolDriver(wchar_t *inf_path, wchar_t *id, UINT lock_timeout)
 {
@@ -973,7 +1131,3 @@ void ShowHtml(HWND hWnd, char *url, wchar_t *option)
 }
 
 #endif
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
