@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Mayaqua Kernel
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Internat.c
@@ -2995,6 +3025,83 @@ wchar_t *UniReplaceFormatStringFor64(wchar_t *fmt)
 	return ret;
 }
 
+// Get lines from a string
+UNI_TOKEN_LIST *UniGetLines(wchar_t *str)
+{
+	UINT i, len;
+	BUF *b = NULL;
+	LIST *o;
+	UNI_TOKEN_LIST *ret;
+	// Validate arguments
+	if (str == NULL)
+	{
+		return UniNullToken();
+	}
+
+	o = NewListFast(NULL);
+
+	len = UniStrLen(str);
+
+	b = NewBuf();
+
+	for (i = 0;i < len;i++)
+	{
+		wchar_t c = str[i];
+		bool f = false;
+
+		if (c == L'\r')
+		{
+			if (str[i + 1] == L'\n')
+			{
+				i++;
+			}
+			f = true;
+		}
+		else if (c == L'\n')
+		{
+			f = true;
+		}
+
+		if (f)
+		{
+			wchar_t zero = 0;
+			wchar_t *s;
+			WriteBuf(b, &zero, sizeof(wchar_t));
+
+			s = (wchar_t *)b->Buf;
+
+			Add(o, UniCopyStr(s));
+
+			ClearBuf(b);
+		}
+		else
+		{
+			WriteBuf(b, &c, sizeof(wchar_t));
+		}
+	}
+
+	if (true)
+	{
+		wchar_t zero = 0;
+		wchar_t *s;
+		WriteBuf(b, &zero, sizeof(wchar_t));
+
+		s = (wchar_t *)b->Buf;
+
+		Add(o, UniCopyStr(s));
+
+		ClearBuf(b);
+	}
+
+	FreeBuf(b);
+
+	ret = UniListToTokenList(o);
+
+	UniFreeStrList(o);
+
+	return ret;
+}
+
 // Display the string on the screen
 void UniPrintStr(wchar_t *string)
 {
@@ -3397,7 +3504,7 @@ UINT UniStrCpy(wchar_t *dst, UINT size, wchar_t *src)
 	}
 	else
 	{
-		len = size / 2 - 1;
+		len = size / sizeof(wchar_t) - 1;
 		Copy(dst, src, len * sizeof(wchar_t));
 		dst[len] = 0;
 	}
@@ -3477,7 +3584,3 @@ UINT UniStrLen(wchar_t *str)
 	return i;
 }
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
