@@ -1,20 +1,21 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Mayaqua Kernel
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
+// Contributors:
+// - nattoheaven (https://github.com/nattoheaven)
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +56,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +101,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Kernel.c
@@ -129,6 +161,7 @@ static int ydays[] =
 };
 
 static UINT current_num_thread = 0;
+static UINT cached_number_of_cpus = 0;
 
 
 
@@ -140,10 +173,113 @@ static LOCALE current_locale;
 LOCK *tick_manual_lock = NULL;
 UINT g_zero = 0;
 
+#define MONSPERYEAR 12
+#define DAYSPERNYEAR 365
+#define DAYSPERLYEAR 366
+#define SECSPERMIN 60
+#define SECSPERHOUR (60*60)
+#define SECSPERDAY (24*60*60)
+#define DAYSPERWEEK 7
+#define TM_SUNDAY	0
+#define TM_MONDAY	1
+#define TM_TUESDAY	2
+#define TM_WEDNESDAY	3
+#define TM_THURSDAY	4
+#define TM_FRIDAY	5
+#define TM_SATURDAY	6
+
+#define TM_YEAR_BASE	1900
+
+#define EPOCH_YEAR	1970
+#define EPOCH_WDAY	TM_THURSDAY
+
+#define isleap(y) (((y) % 4) == 0 && (((y) % 100) != 0 || ((y) % 400) == 0))
+
+static const int	mon_lengths[2][MONSPERYEAR] = {
+	{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
+	{ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
+};
+
+static const int	year_lengths[2] = {
+	DAYSPERNYEAR, DAYSPERLYEAR
+};
+
+
+/*
+ * Taken from FreeBSD src / lib / libc / stdtime / localtime.c 1.43 revision.
+ * localtime.c 7.78.
+ * tzfile.h 1.8
+ * adapted to be replacement gmtime_r.
+ */
+static void
+c_timesub(timep, offset, tmp)
+const time_64t * const			timep;
+const long				offset;
+struct tm * const		tmp;
+{
+	INT64			days;
+	INT64			rem;
+	INT64			y;
+	int			yleap;
+	const int *		ip;
+
+	days = *timep / SECSPERDAY;
+	rem = *timep % SECSPERDAY;
+	rem += (offset);
+	while (rem < 0) {
+		rem += SECSPERDAY;
+		--days;
+	}
+	while (rem >= SECSPERDAY) {
+		rem -= SECSPERDAY;
+		++days;
+	}
+	tmp->tm_hour = (int) (rem / SECSPERHOUR);
+	rem = rem % SECSPERHOUR;
+	tmp->tm_min = (int) (rem / SECSPERMIN);
+	/*
+	** A positive leap second requires a special
+	** representation.  This uses "... ??:59:60" et seq.
+	*/
+	tmp->tm_sec = (int) (rem % SECSPERMIN) ;
+	tmp->tm_wday = (int) ((EPOCH_WDAY + days) % DAYSPERWEEK);
+	if (tmp->tm_wday < 0)
+		tmp->tm_wday += DAYSPERWEEK;
+	y = EPOCH_YEAR;
+#define LEAPS_THRU_END_OF(y)	((y) / 4 - (y) / 100 + (y) / 400)
+	while (days < 0 || days >= (long) year_lengths[yleap = isleap(y)]) {
+		INT64	newy;
+
+		newy = y + days / DAYSPERNYEAR;
+		if (days < 0)
+			--newy;
+		days -= (newy - y) * DAYSPERNYEAR +
+			LEAPS_THRU_END_OF(newy - 1) -
+			LEAPS_THRU_END_OF(y - 1);
+		y = newy;
+	}
+	tmp->tm_year = (int)(y - TM_YEAR_BASE);
+	tmp->tm_yday = (int) days;
+	ip = mon_lengths[yleap];
+	for (tmp->tm_mon = 0; days >= (INT64) ip[tmp->tm_mon]; ++(tmp->tm_mon))
+		days = days - (INT64) ip[tmp->tm_mon];
+	tmp->tm_mday = (int) (days + 1);
+	tmp->tm_isdst = 0;
+}
+
+/*
+* Re-entrant version of gmtime.
+*/
+struct tm * c_gmtime_r(const time_64t* timep, struct tm *tm)
+{
+	c_timesub(timep, 0L, tm);
+	return tm;
+}
+
 // Get the real-time system timer
 UINT TickRealtime()
 {
-#if	defined(OS_WIN32) || defined(CLOCK_REALTIME) || defined(CLOCK_MONOTONIC) || defined(CLOCK_HIGHRES)
+#if	defined(OS_WIN32) || defined(CLOCK_REALTIME) || defined(CLOCK_MONOTONIC) || defined(CLOCK_HIGHRES) || defined(UNIX_MACOS)
 	return Tick() + 1;
 #else
 	return TickRealtimeManual() + 1;
@@ -187,13 +323,57 @@ UINT64 TickGetRealtimeTickValue64()
 
 	gettimeofday(&tv, &tz);
 
-	ret = (UINT64)tv.tv_sec * 1000ULL + (UINT64)tv.tv_usec / 1000ULL;
+	if (sizeof(tv.tv_sec) != 4)
+	{
+		ret = (UINT64)tv.tv_sec * 1000ULL + (UINT64)tv.tv_usec / 1000ULL;
+	}
+	else
+	{
+		ret = (UINT64)((UINT64)((UINT32)tv.tv_sec)) * 1000ULL + (UINT64)tv.tv_usec / 1000ULL;
+	}
 
 	return ret;
 }
 
 #endif	// OS_WIN32
 
+// Get the number of CPUs
+UINT GetNumberOfCpu()
+{
+	UINT ret = 0;
+
+	if (cached_number_of_cpus == 0)
+	{
+		UINT i = 0;
+
+#ifdef	OS_WIN32
+		i = Win32GetNumberOfCpuInner();
+#else	// OS_WIN32
+		i = UnixGetNumberOfCpuInner();
+#endif	// OS_WIN32
+
+		if (i == 0)
+		{
+			i = 8;
+		}
+
+		cached_number_of_cpus = i;
+	}
+
+	ret = cached_number_of_cpus;
+
+	if (ret == 0)
+	{
+		ret = 1;
+	}
+	if (ret > 128)
+	{
+		ret = 128;
+	}
+
+	return ret;
+}
+
 // Creating a thread list
 LIST *NewThreadList()
 {
@@ -287,6 +467,21 @@ void MainteThreadList(LIST *o)
 	UnlockList(o);
 }
 
+// Wait until all threads in the thread list will be stopped
+void WaitAllThreadsWillBeStopped(LIST *o)
+{
+	// Validate arguments
+	if (o == NULL)
+	{
+		return;
+	}
+
+	while (LIST_NUM(o) != 0)
+	{
+		SleepThread(100);
+	}
+}
+
 // Stop all the threads in the thread list
 void StopThreadList(LIST *o)
 {
@@ -675,6 +870,17 @@ void GetDateTimeStrMilli64(char *str, UINT size, UINT64 sec64)
 	UINT64ToSystem(&st, sec64);
 	GetDateTimeStrMilli(str, size, &st);
 }
+void GetDateTimeStrMilli64ForFileName(char *str, UINT size, UINT64 sec64)
+{
+	SYSTEMTIME st;
+	UINT64ToSystem(&st, sec64);
+	GetDateTimeStrMilliForFileName(str, size, &st);
+}
+void GetDateTimeStrMilliForFileName(char *str, UINT size, SYSTEMTIME *tm)
+{
+	Format(str, size, "%04u%02u%02u_%02u%02u%02u",
+		tm->wYear, tm->wMonth, tm->wDay, tm->wHour, tm->wMinute, tm->wSecond);
+}
 void GetDateStr64(char *str, UINT size, UINT64 sec64)
 {
 	SYSTEMTIME st;
@@ -757,7 +963,7 @@ void GetTimeStr64(char *str, UINT size, UINT64 sec64)
 // Convert to a time to be used safely in the current POSIX implementation
 UINT64 SafeTime64(UINT64 sec64)
 {
-	return MAKESURE(sec64, 0, 2115947647000ULL);
+	return MAKESURE(sec64, 0, 4102243323123ULL);
 }
 
 // Thread pool
@@ -933,7 +1139,7 @@ void SetThreadName(UINT thread_id, char *name, void *param)
 		}
 		else
 		{
-			sprintf(tmp, "%s (0x%x)", name, param);
+			sprintf(tmp, "%s (0x%x)", name, (UINT)param);
 		}
 
 		Win32SetThreadName(thread_id, tmp);
@@ -1425,6 +1631,27 @@ void GetDateTimeStrMilli(char *str, UINT size, SYSTEMTIME *st)
 		st->wMilliseconds);
 }
 
+// Get the date and time string in RFC3339 format (example: 2017-09-27T18:25:55.434-9:00)
+void GetDateTimeStrRFC3339(char *str, UINT size, SYSTEMTIME *st, int timezone_min){
+	// Validate arguments
+	if (str == NULL || st == NULL)
+	{
+		return;
+	}
+
+	if(timezone_min == 0){
+		Format(str, size, "%04u-%02u-%02uT%02u:%02u:%02u.%03uZ",
+		st->wYear, st->wMonth, st->wDay,
+		st->wHour, st->wMinute, st->wSecond,
+		st->wMilliseconds);
+	}else{
+		Format(str, size, "%04u-%02u-%02uT%02u:%02u:%02u.%03u%+02d:%02d",
+		st->wYear, st->wMonth, st->wDay,
+		st->wHour, st->wMinute, st->wSecond,
+		st->wMilliseconds, timezone_min/60, timezone_min%60);
+	}
+}
+
 // Get the time string
 void GetSpanStr(char *str, UINT size, UINT64 sec64)
 {
@@ -1636,7 +1863,7 @@ void TmToSystem(SYSTEMTIME *st, struct tm *t)
 	NormalizeTm(&tmp);
 
 	Zero(st, sizeof(SYSTEMTIME));
-	st->wYear = MAKESURE(tmp.tm_year + 1900, 1970, 2037);
+	st->wYear = MAKESURE(tmp.tm_year + 1900, 1970, 2099);
 	st->wMonth = MAKESURE(tmp.tm_mon + 1, 1, 12);
 	st->wDay = MAKESURE(tmp.tm_mday, 1, 31);
 	st->wDayOfWeek = MAKESURE(tmp.tm_wday, 0, 6);
@@ -1656,7 +1883,7 @@ void SystemToTm(struct tm *t, SYSTEMTIME *st)
 	}
 
 	Zero(t, sizeof(struct tm));
-	t->tm_year = MAKESURE(st->wYear, 1970, 2037) - 1900;
+	t->tm_year = MAKESURE(st->wYear, 1970, 2099) - 1900;
 	t->tm_mon = MAKESURE(st->wMonth, 1, 12) - 1;
 	t->tm_mday = MAKESURE(st->wDay, 1, 31);
 	t->tm_hour = MAKESURE(st->wHour, 0, 23);
@@ -1668,7 +1895,7 @@ void SystemToTm(struct tm *t, SYSTEMTIME *st)
 }
 
 // Convert the time_t to SYSTEMTIME
-void TimeToSystem(SYSTEMTIME *st, time_t t)
+void TimeToSystem(SYSTEMTIME *st, time_64t t)
 {
 	struct tm tmp;
 	// Validate arguments
@@ -1682,7 +1909,7 @@ void TimeToSystem(SYSTEMTIME *st, time_t t)
 }
 
 // Convert the time_t to 64-bit SYSTEMTIME
-UINT64 TimeToSystem64(time_t t)
+UINT64 TimeToSystem64(time_64t t)
 {
 	SYSTEMTIME st;
 
@@ -1692,7 +1919,7 @@ UINT64 TimeToSystem64(time_t t)
 }
 
 // Convert the SYSTEMTIME to time_t
-time_t SystemToTime(SYSTEMTIME *st)
+time_64t SystemToTime(SYSTEMTIME *st)
 {
 	struct tm t;
 	// Validate arguments
@@ -1706,7 +1933,7 @@ time_t SystemToTime(SYSTEMTIME *st)
 }
 
 // Convert a 64-bit SYSTEMTIME to a time_t
-time_t System64ToTime(UINT64 i)
+time_64t System64ToTime(UINT64 i)
 {
 	SYSTEMTIME st;
 
@@ -1716,9 +1943,9 @@ time_t System64ToTime(UINT64 i)
 }
 
 // Convert the tm to time_t
-time_t TmToTime(struct tm *t)
+time_64t TmToTime(struct tm *t)
 {
-	time_t tmp;
+	time_64t tmp;
 	// Validate arguments
 	if (t == NULL)
 	{
@@ -1726,7 +1953,7 @@ time_t TmToTime(struct tm *t)
 	}
 
 	tmp = c_mkgmtime(t);
-	if (tmp == (time_t)-1)
+	if (tmp == (time_64t)-1)
 	{
 		return 0;
 	}
@@ -1734,42 +1961,22 @@ time_t TmToTime(struct tm *t)
 }
 
 // Convert time_t to tm
-void TimeToTm(struct tm *t, time_t time)
+void TimeToTm(struct tm *t, time_64t time)
 {
-	struct tm *ret;
 	// Validate arguments
 	if (t == NULL)
 	{
 		return;
 	}
 
-#ifndef	OS_UNIX
-	ret = gmtime(&time);
-#else	// OS_UNIX
-	ret = malloc(sizeof(struct tm));
-	memset(ret, 0, sizeof(ret));
-	gmtime_r(&time, ret);
-#endif	// OS_UNIX
-
-	if (ret == NULL)
-	{
-		Zero(t, sizeof(struct tm));
-	}
-	else
-	{
-		Copy(t, ret, sizeof(struct tm));
-	}
-
-#ifdef	OS_UNIX
-	free(ret);
-#endif	// OS_UNIX
+	Zero(t, sizeof(struct tm));
+	c_gmtime_r(&time, t);
 }
 
 // Normalize the tm
 void NormalizeTm(struct tm *t)
 {
-	struct tm *ret;
-	time_t tmp;
+	time_64t tmp;
 	// Validate arguments
 	if (t == NULL)
 	{
@@ -1777,31 +1984,12 @@ void NormalizeTm(struct tm *t)
 	}
 
 	tmp = c_mkgmtime(t);
-	if (tmp == (time_t)-1)
+	if (tmp == (time_64t)-1)
 	{
 		return;
 	}
 
-#ifndef	OS_UNIX
-	ret = gmtime(&tmp);
-#else	// OS_UNIX
-	ret = malloc(sizeof(struct tm));
-	memset(ret, 0, sizeof(ret));
-	gmtime_r(&tmp, ret);
-#endif	// OS_UNIX
-
-	if (ret == NULL)
-	{
-		Zero(t, sizeof(struct tm));
-	}
-	else
-	{
-		Copy(t, ret, sizeof(struct tm));
-	}
-
-#ifdef	OS_UNIX
-	free(ret);
-#endif	// OS_UNIX
+	c_gmtime_r(&tmp, t);
 }
 
 // Normalize the SYSTEMTIME
@@ -1876,10 +2064,19 @@ INT64 GetTimeDiffEx(SYSTEMTIME *basetime, bool local_time)
 
 	Copy(&snow, basetime, sizeof(SYSTEMTIME));
 
+	if (sizeof(time_t) == 4)
+	{
+		if (snow.wYear >= 2038)
+		{
+			// For old systems: avoid the 2038-year problem
+			snow.wYear = 2037;
+		}
+	}
+
 	SystemToTm(&now, &snow);
 	if (local_time == false)
 	{
-		tmp = c_mkgmtime(&now);
+		tmp = (time_t)c_mkgmtime(&now);
 	}
 	else
 	{
@@ -1891,8 +2088,14 @@ INT64 GetTimeDiffEx(SYSTEMTIME *basetime, bool local_time)
 		return 0;
 	}
 
+#ifndef	OS_UNIX
 	Copy(&t1, localtime(&tmp), sizeof(struct tm));
 	Copy(&t2, gmtime(&tmp), sizeof(struct tm));
+#else	// OS_UNIX
+	localtime_r(&tmp, &t1);
+	gmtime_r(&tmp, &t2);
+#endif	// OS_UNIX
+
 	TmToSystem(&s1, &t1);
 	TmToSystem(&s2, &t2);
 
@@ -1901,48 +2104,12 @@ INT64 GetTimeDiffEx(SYSTEMTIME *basetime, bool local_time)
 	return ret;
 }
 
-// Get the time difference between the local time and system time
-INT64 GetTimeDiff()
-{
-	time_t tmp;
-	struct tm t1, t2;
-	SYSTEMTIME snow;
-	struct tm now;
-	SYSTEMTIME s1, s2;
-	INT64 ret;
-
-	static INT64 cache = INFINITE;
-
-	if (cache != INFINITE)
-	{
-		// Returns the cache data after measured once
-		return cache;
-	}
-
-	SystemTime(&snow);
-	SystemToTm(&now, &snow);
-	tmp = c_mkgmtime(&now);
-	if (tmp == (time_t)-1)
-	{
-		return 0;
-	}
-
-	Copy(&t1, localtime(&tmp), sizeof(struct tm));
-	Copy(&t2, gmtime(&tmp), sizeof(struct tm));
-	TmToSystem(&s1, &t1);
-	TmToSystem(&s2, &t2);
-
-	cache = ret = (INT)SystemToUINT64(&s1) - (INT)SystemToUINT64(&s2);
-
-	return ret;
-}
-
 // Convert UINT64 to the SYSTEMTIME
 void UINT64ToSystem(SYSTEMTIME *st, UINT64 sec64)
 {
 	UINT64 tmp64;
 	UINT sec, millisec;
-	time_t time;
+	time_64t time;
 	// Validate arguments
 	if (st == NULL)
 	{
@@ -1953,7 +2120,7 @@ void UINT64ToSystem(SYSTEMTIME *st, UINT64 sec64)
 	tmp64 = sec64 / (UINT64)1000;
 	millisec = (UINT)(sec64 - tmp64 * (UINT64)1000);
 	sec = (UINT)tmp64;
-	time = (time_t)sec;
+	time = (time_64t)sec;
 	TimeToSystem(st, time);
 	st->wMilliseconds = (WORD)millisec;
 }
@@ -1962,7 +2129,7 @@ void UINT64ToSystem(SYSTEMTIME *st, UINT64 sec64)
 UINT64 SystemToUINT64(SYSTEMTIME *st)
 {
 	UINT64 sec64;
-	time_t time;
+	time_64t time;
 	// Validate arguments
 	if (st == NULL)
 	{
@@ -2021,7 +2188,7 @@ void SystemTime(SYSTEMTIME *st)
 	KS_INC(KS_GETTIME_COUNT);
 }
 
-time_t c_mkgmtime(struct tm *tm)
+time_64t c_mkgmtime(struct tm *tm)
 {
 	int years, months, days, hours, minutes, seconds;
 
@@ -2072,7 +2239,7 @@ time_t c_mkgmtime(struct tm *tm)
 		tm->tm_isdst = 0;
 
 		if (years < 1970)
-			return (time_t)-1;
+			return (time_64t)-1;
 
 #if (defined(TM_YEAR_MAX) && defined(TM_MON_MAX) && defined(TM_MDAY_MAX))
 #if (defined(TM_HOUR_MAX) && defined(TM_MIN_MAX) && defined(TM_SEC_MAX))
@@ -2086,11 +2253,11 @@ time_t c_mkgmtime(struct tm *tm)
 			(hours == TM_HOUR_MAX &&
 			(minutes > TM_MIN_MAX ||
 			(minutes == TM_MIN_MAX && seconds > TM_SEC_MAX) )))))))
-			return (time_t)-1;
+			return (time_64t)-1;
 #endif
 #endif
 
-		return (time_t)(86400L * (unsigned long)(unsigned)days +
+		return (time_64t)(86400L * (unsigned long)(unsigned)days +
 			3600L * (unsigned long)hours +
 			(unsigned long)(60 * minutes + seconds));
 }
@@ -2166,7 +2333,3 @@ void AbortExitEx(char *msg)
 #endif	// OS_WIN32
 }
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
