@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Mayaqua Kernel
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // TcpIp.c
@@ -144,14 +174,14 @@ ICMP_RESULT *IcmpParseResult(IP *dest_ip, USHORT src_id, USHORT src_seqno, UCHAR
 	if (true)
 	{
 		UINT ip_header_size = GetIpHeaderSize(recv_buffer, i);
-		if (ip_header_size >= sizeof(IPV4_HEADER))
+		if (ip_header_size >= sizeof(IPV4_HEADER) && (ip_header_size <= i))
 		{
 			IPV4_HEADER *ipv4 = (IPV4_HEADER *)recv_buffer;
 			if ((IPV4_GET_VERSION(ipv4) == 4) && (ipv4->Protocol == IP_PROTO_ICMPV4))
 			{
 				UINT ip_total_len = (UINT)Endian16(ipv4->TotalLength);
 
-				if ((ip_total_len >= sizeof(IPV4_HEADER)) && (ip_total_len <= i))
+				if ((ip_total_len >= sizeof(IPV4_HEADER)) && (ip_total_len <= i) && (ip_total_len >= ip_header_size))
 				{
 					UINT icmp_packet_size = ip_total_len - ip_header_size;
 					ICMP_HEADER *icmp = (ICMP_HEADER *)(recv_buffer + ip_header_size);
@@ -639,6 +669,7 @@ bool AdjustTcpMssL3(UCHAR *src, UINT src_size, UINT mss)
 	if (ip_ver == 4)
 	{
 		UINT ip_header_size;
+		UINT ip_total_length;
 		// IPv4
 		if (src_size < sizeof(IPV4_HEADER))
 		{
@@ -679,8 +710,22 @@ bool AdjustTcpMssL3(UCHAR *src, UINT src_size, UINT mss)
 			return false;
 		}
 
+		ip_total_length = READ_USHORT(&ip->TotalLength);
+
+		if (ip_total_length < ip_header_size)
+		{
+			// Invalid total length
+			return false;
+		}
+
+		if (src_size < ip_total_length)
+		{
+			// No total length
+			return false;
+		}
+
 		src += ip_header_size;
-		src_size -= ip_header_size;
+		src_size = ip_total_length - ip_header_size;
 
 		if (src_size < sizeof(TCP_HEADER))
 		{
@@ -1643,6 +1688,10 @@ PKT *ClonePacket(PKT *p, bool copy_data)
 		ret->L7.IkeHeader = MallocFast(sizeof(IKE_HEADER));
 		Copy(ret->L7.IkeHeader, p->L7.IkeHeader, sizeof(IKE_HEADER));
 		break;
+
+	case L7_DNS:
+		StrCpy(ret->DnsQueryHost, sizeof(ret->DnsQueryHost), p->DnsQueryHost);
+		break;
 	}
 
 	// Address data
@@ -1798,19 +1847,28 @@ PKT *ParsePacketEx4(UCHAR *buf, UINT size, bool no_l3, UINT vlan_type_id, bool b
 	if (no_http == false)
 	{
 		USHORT port_raw = Endian16(80);
+		USHORT port_raw2 = Endian16(8080);
+		USHORT port_raw3 = Endian16(443);
+		USHORT port_raw4 = Endian16(3128);
 
 		// Analyze if the packet is a part of HTTP
 		if ((p->TypeL3 == L3_IPV4 || p->TypeL3 == L3_IPV6) && p->TypeL4 == L4_TCP)
 		{
 			TCP_HEADER *tcp = p->L4.TCPHeader;
-			if (tcp->DstPort == port_raw)
+			if (tcp != NULL && (tcp->DstPort == port_raw || tcp->DstPort == port_raw2 || tcp->DstPort == port_raw4) &&
+				(!((tcp->Flag & TCP_SYN) || (tcp->Flag & TCP_RST) || (tcp->Flag & TCP_FIN))))
 			{
-				if (tcp != NULL && (!((tcp->Flag & TCP_SYN) || (tcp->Flag & TCP_RST) || (tcp->Flag & TCP_FIN))))
+				if (p->PayloadSize >= 1)
 				{
-					if (p->PayloadSize >= 1)
-					{
-						p->HttpLog = ParseHttpAccessLog(p);
-					}
+					p->HttpLog = ParseHttpAccessLog(p);
+				}
+			}
+			if (tcp != NULL && tcp->DstPort == port_raw3 &&
+				(!((tcp->Flag & TCP_SYN) || (tcp->Flag & TCP_RST) || (tcp->Flag & TCP_FIN))))
+			{
+				if (p->PayloadSize >= 1)
+				{
+					p->HttpLog = ParseHttpsAccessLog(p);
 				}
 			}
 		}
@@ -1899,7 +1957,7 @@ void CorrectChecksum(PKT *p)
 						{
 							udp->Checksum = 0;
 
-							if ((IPV4_GET_FLAGS(v4) & 0x01) == 0)
+							if ((IPV4_GET_FLAGS(v4) & 0x01) == 0 && (p->IPv4PayloadSize >= udp_len))
 							{
 								// Calculate the checksum correctly based on the data in case of a non-fragmented packet
 								udp->Checksum = CalcChecksumForIPv4(v4->SrcIP, v4->DstIP, IP_PROTO_UDP, udp, udp_len, 0);
@@ -1965,7 +2023,7 @@ void CorrectChecksum(PKT *p)
 						{
 							udp->Checksum = 0;
 
-							if (v6info->FragmentHeader == NULL || ((IPV6_GET_FLAGS(v6info->FragmentHeader) & IPV6_FRAGMENT_HEADER_FLAG_MORE_FRAGMENTS) == 0))
+							if ((v6info->FragmentHeader == NULL || ((IPV6_GET_FLAGS(v6info->FragmentHeader) & IPV6_FRAGMENT_HEADER_FLAG_MORE_FRAGMENTS) == 0)) && (v6info->PayloadSize >= udp_len))
 							{
 								// If the packet is not fragmented, recalculate the checksum
 								udp->Checksum = CalcChecksumForIPv6(&v6->SrcAddress, &v6->DestAddress, IP_PROTO_UDP, udp, udp_len, 0);
@@ -1983,6 +2041,33 @@ void CorrectChecksum(PKT *p)
 }
 
 
+// Parse the HTTPS access log
+HTTPLOG *ParseHttpsAccessLog(PKT *pkt)
+{
+	HTTPLOG h;
+	char sni[MAX_PATH];
+	// Validate arguments
+	if (pkt == NULL)
+	{
+		return NULL;
+	}
+
+	if (GetSniNameFromSslPacket(pkt->Payload, pkt->PayloadSize, sni, sizeof(sni)) == false)
+	{
+		return NULL;
+	}
+
+	Zero(&h, sizeof(h));
+
+	StrCpy(h.Method, sizeof(h.Method), "SSL_Connect");
+	StrCpy(h.Hostname, sizeof(h.Hostname), sni);
+	h.Port = Endian16(pkt->L4.TCPHeader->DstPort);
+	StrCpy(h.Path, sizeof(h.Path), "/");
+	h.IsSsl = true;
+
+	return Clone(&h, sizeof(h));
+}
+
 // Parse the HTTP access log
 HTTPLOG *ParseHttpAccessLog(PKT *pkt)
 {
@@ -2152,7 +2237,7 @@ bool ParsePacketL2Ex(PKT *p, UCHAR *buf, UINT size, bool no_l3)
 			b2 = false;
 		}
 	}
-	if (b1 || b2 || (Cmp(p->MacHeader->SrcAddress, p->MacHeader->DestAddress, 6) == 0))
+	if (b1 || b2 || (memcmp(p->MacHeader->SrcAddress, p->MacHeader->DestAddress, 6) == 0))
 	{
 		p->InvalidSourcePacket = true;
 	}
@@ -2783,6 +2868,7 @@ PKT *ParsePacketIPv4WithDummyMacHeader(UCHAR *buf, UINT size)
 {
 	UCHAR *tmp;
 	UINT tmp_size;
+	PKT *ret;
 	// Validate arguments
 	if (buf == NULL)
 	{
@@ -2795,7 +2881,14 @@ PKT *ParsePacketIPv4WithDummyMacHeader(UCHAR *buf, UINT size)
 	WRITE_USHORT(tmp + 12, MAC_PROTO_IPV4);
 	Copy(tmp + 14, buf, size);
 
-	return ParsePacket(tmp, tmp_size);
+	ret = ParsePacket(tmp, tmp_size);
+
+	if (ret == NULL)
+	{
+		Free(tmp);
+	}
+
+	return ret;
 }
 
 // IPv4 parsing
@@ -2843,6 +2936,7 @@ bool ParsePacketIPv4(PKT *p, UCHAR *buf, UINT size)
 	{
 		// Quit analysing since this is fragmented
 		p->TypeL4 = L4_FRAGMENT;
+
 		return true;
 	}
 
@@ -2929,6 +3023,148 @@ bool ParseTCP(PKT *p, UCHAR *buf, UINT size)
 	return true;
 }
 
+// Get the next byte
+UCHAR GetNextByte(BUF *b)
+{
+	UCHAR c = 0;
+	// Validate arguments
+	if (b == NULL)
+	{
+		return 0;
+	}
+
+	if (ReadBuf(b, &c, 1) != 1)
+	{
+		return 0;
+	}
+
+	return c;
+}
+
+// Interpret the DNS query
+bool ParseDnsQuery(char *name, UINT name_size, void *data, UINT data_size)
+{
+	BUF *b;
+	char tmp[257];
+	bool ok = true;
+	USHORT val;
+	// Validate arguments
+	if (name == NULL || data == NULL || data_size == 0)
+	{
+		return false;
+	}
+	StrCpy(name, name_size, "");
+
+	b = NewBuf();
+	WriteBuf(b, data, data_size);
+	SeekBuf(b, 0, 0);
+
+	while (true)
+	{
+		UINT next_len = (UINT)GetNextByte(b);
+		if (next_len > 0)
+		{
+			// Read only the specified length
+			Zero(tmp, sizeof(tmp));
+			if (ReadBuf(b, tmp, next_len) != next_len)
+			{
+				ok = false;
+				break;
+			}
+			// Append
+			if (StrLen(name) != 0)
+			{
+				StrCat(name, name_size, ".");
+			}
+			StrCat(name, name_size, tmp);
+		}
+		else
+		{
+			// Read all
+			break;
+		}
+	}
+
+	if (ReadBuf(b, &val, sizeof(val)) != sizeof(val))
+	{
+		ok = false;
+	}
+	else
+	{
+		if (Endian16(val) != 0x01 && Endian16(val) != 0x0c)
+		{
+			ok = false;
+		}
+	}
+
+	if (ReadBuf(b, &val, sizeof(val)) != sizeof(val))
+	{
+		ok = false;
+	}
+	else
+	{
+		if (Endian16(val) != 0x01)
+		{
+			ok = false;
+		}
+	}
+
+	FreeBuf(b);
+
+	if (ok == false || StrLen(name) == 0)
+	{
+		return false;
+	}
+	else
+	{
+		return true;
+	}
+}
+
+// DNS parsing
+void ParseDNS(PKT *p, UCHAR *buf, UINT size)
+{
+	UCHAR *query_data;
+	UINT query_data_size;
+	DNSV4_HEADER *dns;
+	char hostname[MAX_SIZE];
+	if (p == NULL|| buf == NULL)
+	{
+		return;
+	}
+
+	if (size < sizeof(DNSV4_HEADER))
+	{
+		return;
+	}
+
+	dns = (DNSV4_HEADER *)buf;
+
+	if ((dns->Flag1 & 78) != 0 || (dns->Flag1 & 0x80) != 0)
+	{
+		// Illegal opcode
+		return;
+	}
+	if (Endian16(dns->NumQuery) != 1)
+	{
+		// Number of queries is invalid
+		return;
+	}
+
+	query_data = ((UCHAR *)dns) + sizeof(DNSV4_HEADER);
+	query_data_size = size - sizeof(DNSV4_HEADER);
+
+	// Interpret the query
+	if (ParseDnsQuery(hostname, sizeof(hostname), query_data, query_data_size) == false)
+	{
+		// Interpretation fails
+		return;
+	}
+
+	StrCpy(p->DnsQueryHost, sizeof(p->DnsQueryHost), hostname);
+	p->TypeL7 = L7_DNS;
+}
+
 // UDP parsing
 bool ParseUDP(PKT *p, UCHAR *buf, UINT size)
 {
@@ -2972,6 +3208,12 @@ bool ParseUDP(PKT *p, UCHAR *buf, UINT size)
 		}
 	}
 
+	if (dst_port == 53)
+	{
+		ParseDNS(p, buf, size);
+		return true;
+	}
+
 	if (src_port == 500 || dst_port == 500 || src_port == 4500 || dst_port == 4500)
 	{
 		if (p->PayloadSize >= sizeof(IKE_HEADER))
@@ -3273,11 +3515,44 @@ BUF *BuildDhcpOptionsBuf(LIST *o)
 	for (i = 0;i < LIST_NUM(o);i++)
 	{
 		DHCP_OPTION *d = LIST_DATA(o, i);
+		UINT current_size = d->Size;
+		UINT current_pos = 0;
+
 		id = (UCHAR)d->Id;
-		sz = (UCHAR)d->Size;
+		if (d->Size <= 255)
+		{
+			sz = (UCHAR)d->Size;
+		}
+		else
+		{
+			sz = 0xFF;
+		}
 		WriteBuf(b, &id, 1);
 		WriteBuf(b, &sz, 1);
-		WriteBuf(b, d->Data, d->Size);
+		WriteBuf(b, d->Data, sz);
+
+		current_size -= sz;
+		current_pos += sz;
+
+		while (current_size != 0)
+		{
+			id = DHCP_ID_PRIVATE;
+			if (current_size <= 255)
+			{
+				sz = (UCHAR)current_size;
+			}
+			else
+			{
+				sz = 0xFF;
+			}
+			WriteBuf(b, &id, 1);
+			WriteBuf(b, &sz, 1);
+			WriteBuf(b, ((UCHAR *)d->Data) + current_pos, sz);
+
+			current_size -= sz;
+			current_pos += sz;
+		}
+
 	}
 
 	id = 0xff;
@@ -3341,6 +3616,19 @@ LIST *BuildDhcpOption(DHCP_OPTION_LIST *opt)
 
 	FreeBuf(dns_buf);
 
+	if (opt->ClasslessRoute.NumExistingRoutes >= 1)
+	{
+		BUF *b = DhcpBuildClasslessRouteData(&opt->ClasslessRoute);
+
+		if (b != NULL)
+		{
+			Add(o, NewDhcpOption(DHCP_ID_CLASSLESS_ROUTE, b->Buf, b->Size));
+			Add(o, NewDhcpOption(DHCP_ID_MS_CLASSLESS_ROUTE, b->Buf, b->Size));
+
+			FreeBuf(b);
+		}
+	}
+
 	return o;
 }
 
@@ -3477,6 +3765,20 @@ DHCP_OPTION_LIST *ParseDhcpOptionList(void *data, UINT size)
 			}
 		}
 
+		// Classless static routing table entries
+		// RFC 3442
+		a = GetDhcpOption(o, DHCP_ID_CLASSLESS_ROUTE);
+		if (a != NULL)
+		{
+			DhcpParseClasslessRouteData(&ret->ClasslessRoute, a->Data, a->Size);
+		}
+		// Microsoft Extension
+		a = GetDhcpOption(o, DHCP_ID_MS_CLASSLESS_ROUTE);
+		if (a != NULL)
+		{
+			DhcpParseClasslessRouteData(&ret->ClasslessRoute, a->Data, a->Size);
+		}
+
 		break;
 	}
 
@@ -3486,6 +3788,340 @@ DHCP_OPTION_LIST *ParseDhcpOptionList(void *data, UINT size)
 	return ret;
 }
 
+// Normalize the classless routing table string
+bool NormalizeClasslessRouteTableStr(char *dst, UINT dst_size, char *src)
+{
+	DHCP_CLASSLESS_ROUTE_TABLE t;
+	// Validate arguments
+	if (dst == NULL || src == NULL)
+	{
+		return false;
+	}
+
+	Zero(&t, sizeof(t));
+	if (ParseClasslessRouteTableStr(&t, src))
+	{
+		BuildClasslessRouteTableStr(dst, dst_size, &t);
+
+		return true;
+	}
+
+	return false;
+}
+
+// Build the string from the classless routing table
+void BuildClasslessRouteTableStr(char *str, UINT str_size, DHCP_CLASSLESS_ROUTE_TABLE *t)
+{
+	UINT i;
+	UINT num = 0;
+	ClearStr(str, str_size);
+	// Validate arguments
+	if (str == NULL || t == NULL)
+	{
+		return;
+	}
+
+	for (i = 0;i < MAX_DHCP_CLASSLESS_ROUTE_ENTRIES;i++)
+	{
+		DHCP_CLASSLESS_ROUTE *r = &t->Entries[i];
+
+		if (r->Exists)
+		{
+			char tmp[128];
+
+			Zero(tmp, sizeof(tmp));
+			BuildClasslessRouteStr(tmp, sizeof(tmp), r);
+
+			if (IsEmptyStr(tmp) == false)
+			{
+				if (num >= 1)
+				{
+					StrCat(str, str_size, ", ");
+				}
+
+				StrCat(str, str_size, tmp);
+
+				num++;
+			}
+		}
+	}
+}
+
+// Build the string from the classless routing table entry
+void BuildClasslessRouteStr(char *str, UINT str_size, DHCP_CLASSLESS_ROUTE *r)
+{
+	ClearStr(str, str_size);
+	// Validate arguments
+	if (str == NULL || r == NULL || r->Exists == false)
+	{
+		return;
+	}
+
+	Format(str, str_size, "%r/%r/%r", &r->Network, &r->SubnetMask, &r->Gateway);
+}
+
+// Check the classless routing table string
+bool CheckClasslessRouteTableStr(char *str)
+{
+	DHCP_CLASSLESS_ROUTE_TABLE d;
+
+	// Validate arguments
+	if (str == NULL)
+	{
+		return false;
+	}
+
+	return ParseClasslessRouteTableStr(&d, str);
+}
+
+// Parse the classless routing table string
+bool ParseClasslessRouteTableStr(DHCP_CLASSLESS_ROUTE_TABLE *d, char *str)
+{
+	bool ret = true;
+	TOKEN_LIST *t;
+	// Validate arguments
+	if (d == NULL || str == NULL)
+	{
+		return false;
+	}
+
+	Zero(d, sizeof(DHCP_CLASSLESS_ROUTE_TABLE));
+
+	t = ParseTokenWithoutNullStr(str, NULL);
+
+	if (t != NULL)
+	{
+		UINT i;
+
+		for (i = 0;i < t->NumTokens;i++)
+		{
+			DHCP_CLASSLESS_ROUTE r;
+
+			Zero(&r, sizeof(r));
+			if (ParseClasslessRouteStr(&r, t->Token[i]))
+			{
+				if (d->NumExistingRoutes < MAX_DHCP_CLASSLESS_ROUTE_ENTRIES)
+				{
+					Copy(&d->Entries[d->NumExistingRoutes], &r, sizeof(DHCP_CLASSLESS_ROUTE));
+					d->NumExistingRoutes++;
+				}
+				else
+				{
+					// Overflow
+					ret = false;
+					break;
+				}
+			}
+			else
+			{
+				// Parse error
+				ret = false;
+				break;
+			}
+		}
+	}
+
+	FreeToken(t);
+
+	return ret;
+}
+
+// Parse the classless routing table entry string
+bool ParseClasslessRouteStr(DHCP_CLASSLESS_ROUTE *r, char *str)
+{
+	TOKEN_LIST *t;
+	bool ret = false;
+	char tmp[MAX_PATH];
+	// Validate arguments
+	if (r == NULL || str == NULL)
+	{
+		return false;
+	}
+
+	StrCpy(tmp, sizeof(tmp), str);
+	Trim(tmp);
+
+	t = ParseTokenWithoutNullStr(str, "/");
+	if (t == NULL)
+	{
+		return false;
+	}
+
+	if (t->NumTokens == 3)
+	{
+		char ip_and_mask[MAX_PATH];
+		char gateway[MAX_PATH];
+
+		Zero(r, sizeof(DHCP_CLASSLESS_ROUTE));
+
+		Format(ip_and_mask, sizeof(ip_and_mask), "%s/%s", t->Token[0], t->Token[1]);
+		StrCpy(gateway, sizeof(gateway), t->Token[2]);
+
+		if (ParseIpAndSubnetMask46(ip_and_mask, &r->Network, &r->SubnetMask))
+		{
+			r->SubnetMaskLen = SubnetMaskToInt4(&r->SubnetMask);
+
+			if (StrToIP(&r->Gateway, gateway))
+			{
+				if (IsIP4(&r->Gateway) && IsIP4(&r->Network) && IsIP4(&r->SubnetMask))
+				{
+					r->Exists = true;
+
+					IPAnd4(&r->Network, &r->Network, &r->SubnetMask);
+
+					ret = true;
+				}
+			}
+		}
+	}
+
+	FreeToken(t);
+
+	return ret;
+}
+
+// Build the classless static routing table data for a DHCP message
+BUF *DhcpBuildClasslessRouteData(DHCP_CLASSLESS_ROUTE_TABLE *t)
+{
+	BUF *b;
+	UINT i;
+	// Validate arguments
+	if (t == NULL || t->NumExistingRoutes == 0)
+	{
+		return NULL;
+	}
+
+	b = NewBuf();
+
+	for (i = 0;i < MAX_DHCP_CLASSLESS_ROUTE_ENTRIES;i++)
+	{
+		DHCP_CLASSLESS_ROUTE *r = &t->Entries[i];
+
+		if (r->Exists && r->SubnetMaskLen <= 32)
+		{
+			UCHAR c;
+			UINT data_len;
+			UINT ip32;
+			UCHAR tmp[4];
+
+			// Width of subnet mask
+			c = (UCHAR)r->SubnetMaskLen;
+			WriteBuf(b, &c, 1);
+
+			// Number of significant octets
+			data_len = (r->SubnetMaskLen + 7) / 8;
+			Zero(tmp, sizeof(tmp));
+			Copy(tmp, &r->Network, data_len);
+			WriteBuf(b, tmp, data_len);
+
+			// Gateway
+			ip32 = IPToUINT(&r->Gateway);
+			WriteBuf(b, &ip32, sizeof(UINT));
+		}
+	}
+
+	SeekBufToBegin(b);
+
+	return b;
+}
+
+// Parse a classless static routing table entries from the DHCP message
+void DhcpParseClasslessRouteData(DHCP_CLASSLESS_ROUTE_TABLE *t, void *data, UINT size)
+{
+	BUF *b;
+	// Validate arguments
+	if (t == NULL || data == NULL || size == 0)
+	{
+		return;
+	}
+
+	b = MemToBuf(data, size);
+
+	while (b->Current < b->Size)
+	{
+		UCHAR c;
+		UINT subnet_mask_len;
+		UINT data_len;
+		UCHAR tmp[4];
+		IP ip;
+		IP mask;
+		IP gateway;
+		DHCP_CLASSLESS_ROUTE r;
+		UINT ip32;
+		bool exists = false;
+		UINT i;
+
+		// Subnet mask length
+		c = ReadBufChar(b);
+		subnet_mask_len = c;
+		if (subnet_mask_len > 32)
+		{
+			// Invalid data
+			break;
+		}
+
+		data_len = (subnet_mask_len + 7) / 8;
+		if (data_len > 4)
+		{
+			// Invalid data
+			break;
+		}
+
+		Zero(tmp, sizeof(tmp));
+		if (ReadBuf(b, tmp, data_len) != data_len)
+		{
+			// Invalid data
+			break;
+		}
+
+		// IP address body
+		Zero(&ip, sizeof(IP));
+		Copy(ip.addr, tmp, data_len);
+
+		Zero(&mask, sizeof(mask));
+		IntToSubnetMask4(&mask, subnet_mask_len);
+
+		// Gateway address
+		Zero(&gateway, sizeof(gateway));
+		if (ReadBuf(b, &ip32, sizeof(UINT)) != sizeof(UINT))
+		{
+			// Invalid data
+			break;
+		}
+		UINTToIP(&gateway, ip32);
+
+		Zero(&r, sizeof(r));
+		r.Exists = true;
+		Copy(&r.Gateway, &gateway, sizeof(IP));
+		Copy(&r.Network, &ip, sizeof(IP));
+		Copy(&r.SubnetMask, &mask, sizeof(IP));
+		r.SubnetMaskLen = subnet_mask_len;
+
+		for (i = 0;i < MAX_DHCP_CLASSLESS_ROUTE_ENTRIES;i++)
+		{
+			if (Cmp(&t->Entries[i], &r, sizeof(DHCP_CLASSLESS_ROUTE)) == 0)
+			{
+				exists = true;
+				break;
+			}
+		}
+
+		if (exists == false)
+		{
+			if (t->NumExistingRoutes >= MAX_DHCP_CLASSLESS_ROUTE_ENTRIES)
+			{
+				// Overflow
+				break;
+			}
+
+			Copy(&t->Entries[t->NumExistingRoutes], &r, sizeof(DHCP_CLASSLESS_ROUTE));
+			t->NumExistingRoutes++;
+		}
+	}
+
+	FreeBuf(b);
+}
+
 // Finding a DHCP option
 DHCP_OPTION *GetDhcpOption(LIST *o, UINT id)
 {
@@ -3509,6 +4145,42 @@ DHCP_OPTION *GetDhcpOption(LIST *o, UINT id)
 	return ret;
 }
 
+// Get the best classless routing table entry from the routing table
+DHCP_CLASSLESS_ROUTE *GetBestClasslessRoute(DHCP_CLASSLESS_ROUTE_TABLE *t, IP *ip)
+{
+	DHCP_CLASSLESS_ROUTE *ret = NULL;
+	UINT i;
+	UINT max_mask = 0;
+	// Validate arguments
+	if (t == NULL || ip == NULL)
+	{
+		return NULL;
+	}
+	if (t->NumExistingRoutes == 0)
+	{
+		return NULL;
+	}
+
+	for (i = 0;i < MAX_DHCP_CLASSLESS_ROUTE_ENTRIES;i++)
+	{
+		DHCP_CLASSLESS_ROUTE *e = &t->Entries[i];
+
+		if (e->Exists)
+		{
+			if (IsInSameNetwork4(ip, &e->Network, &e->SubnetMask))
+			{
+				if (max_mask <= e->SubnetMaskLen)
+				{
+					max_mask = e->SubnetMaskLen;
+					ret = e;
+				}
+			}
+		}
+	}
+
+	return ret;
+}
+
 // Release the DHCP option
 void FreeDhcpOptions(LIST *o)
 {
@@ -3534,6 +4206,7 @@ LIST *ParseDhcpOptions(void *data, UINT size)
 {
 	BUF *b;
 	LIST *o;
+	DHCP_OPTION *last_opt;
 	// Validate arguments
 	if (data == NULL)
 	{
@@ -3546,6 +4219,8 @@ LIST *ParseDhcpOptions(void *data, UINT size)
 
 	o = NewListFast(NULL);
 
+	last_opt = NULL;
+
 	while (true)
 	{
 		UCHAR c = 0;
@@ -3564,12 +4239,27 @@ LIST *ParseDhcpOptions(void *data, UINT size)
 			break;
 		}
 
-		opt = ZeroMalloc(sizeof(DHCP_OPTION));
-		opt->Id = (UINT)c;
-		opt->Size = (UINT)sz;
-		opt->Data = ZeroMalloc((UINT)sz);
-		ReadBuf(b, opt->Data, sz);
-		Add(o, opt);
+		if (c == DHCP_ID_PRIVATE && last_opt != NULL)
+		{
+			UINT new_size = last_opt->Size + (UINT)sz;
+			UCHAR *new_buf = ZeroMalloc(new_size);
+			Copy(new_buf, last_opt->Data, last_opt->Size);
+			ReadBuf(b, new_buf + last_opt->Size, sz);
+			Free(last_opt->Data);
+			last_opt->Data = new_buf;
+			last_opt->Size = new_size;
+		}
+		else
+		{
+			opt = ZeroMalloc(sizeof(DHCP_OPTION));
+			opt->Id = (UINT)c;
+			opt->Size = (UINT)sz;
+			opt->Data = ZeroMalloc((UINT)sz);
+			ReadBuf(b, opt->Data, sz);
+			Add(o, opt);
+
+			last_opt = opt;
+		}
 	}
 
 	FreeBuf(b);
@@ -3796,7 +4486,3 @@ LABEL_CLEANUP:
 		return NULL;
 	}
 }
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
