@@ -1,20 +1,22 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Mayaqua Kernel
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Authors: Daiyuu Nobori
+// Contributors:
+// - Melvyn (https://github.com/yaurthek)
+// - nattoheaven (https://github.com/nattoheaven)
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +57,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +102,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Unix.c
@@ -94,8 +127,11 @@
 #include <errno.h>
 #include <Mayaqua/Mayaqua.h>
 
-// Struct statfs for MacOS X
 #ifdef	UNIX_MACOS
+#include <mach/clock.h>
+#include <mach/mach.h>
+#ifdef	NO_VLAN
+// Struct statfs for MacOS X
 typedef struct fsid { int32_t val[2]; } fsid_t;
 struct statfs {
         short   f_otype;                /* TEMPORARY SHADOW COPY OF f_type */
@@ -117,6 +153,9 @@ struct statfs {
         char    f_mntonname[90];  /* directory on which mounted */
         char    f_mntfromname[90];/* mounted filesystem */
 };
+#else	// NO_VLAN
+#include <sys/mount.h>
+#endif	// NO_VLAN
 #endif	// UNIX_MACOS
 
 // Scandir() function for Solaris
@@ -927,6 +966,24 @@ void *UnixNewSingleInstance(char *instance_name)
 	}
 }
 
+// Set the high oom score
+void UnixSetHighOomScore()
+{
+	IO *o;
+	char tmp[256];
+
+	sprintf(tmp, "/proc/%u/oom_score_adj", getpid());
+
+	o = UnixFileCreate(tmp);
+	if (o != NULL)
+	{
+		char tmp[128];
+		sprintf(tmp, "%u\n", 800);
+		UnixFileWrite(o, tmp, strlen(tmp));
+		UnixFileClose(o, false);
+	}
+}
+
 // Raise the priority of the process
 void UnixSetHighPriority()
 {
@@ -953,6 +1010,63 @@ void UnixRestorePriority()
 	}
 }
 
+UINT UnixGetNumberOfCpuInner()
+{
+	BUF *b;
+	UINT ret = 0;
+
+	b = ReadDump("/proc/cpuinfo");
+	if (b != NULL)
+	{
+		while (true)
+		{
+			char *line = CfgReadNextLine(b);
+
+			if (line == NULL)
+			{
+				break;
+			}
+
+			if (IsEmptyStr(line) == false)
+			{
+				TOKEN_LIST *t = ParseToken(line, ":");
+				if (t != NULL)
+				{
+					if (t->NumTokens >= 2)
+					{
+						char *key = t->Token[0];
+						char *value = t->Token[1];
+
+						Trim(key);
+						Trim(value);
+
+						if (StrCmpi(key, "processor") == 0)
+						{
+							if (IsNum(value))
+							{
+								UINT i = ToInt(value) + 1;
+
+								if (i >= 1 && i <= 128)
+								{
+									ret = MAX(ret, i);
+								}
+							}
+						}
+					}
+
+					FreeToken(t);
+				}
+			}
+
+			Free(line);
+		}
+
+		FreeBuf(b);
+	}
+
+	return ret;
+}
+
 // Get the product ID
 char *UnixGetProductId()
 {
@@ -1054,7 +1168,7 @@ void UnixGetOsInfo(OS_INFO *info)
 			}
 			else
 			{
-				info->OsVersion = CopyStr("Unknown Liunx Version");
+				info->OsVersion = CopyStr("Unknown Linux Version");
 				info->OsVendorName = CopyStr("Unknown Vendor");
 			}
 		}
@@ -1974,6 +2088,7 @@ void UnixInc32(UINT *value)
 void UnixGetSystemTime(SYSTEMTIME *system_time)
 {
 	time_t now = 0;
+	time_64t now2 = 0;
 	struct tm tm;
 	struct timeval tv;
 	struct timezone tz;
@@ -1991,7 +2106,16 @@ void UnixGetSystemTime(SYSTEMTIME *system_time)
 
 	time(&now);
 
-	gmtime_r(&now, &tm);
+	if (sizeof(time_t) == 4)
+	{
+		now2 = (time_64t)((UINT64)((UINT32)now));
+	}
+	else
+	{
+		now2 = now;
+	}
+
+	c_gmtime_r(&now2, &tm);
 
 	TmToSystem(system_time, &tm);
 
@@ -2030,7 +2154,7 @@ UINT64 UnixGetTick64()
 #endif	// CLOCK_MONOTONIC
 #endif	// CLOCK_HIGHRES
 
-	ret = (UINT64)t.tv_sec * 1000LL + (UINT64)t.tv_nsec / 1000000LL;
+	ret = ((UINT64)((UINT32)t.tv_sec)) * 1000LL + (UINT64)t.tv_nsec / 1000000LL;
 
 	if (akirame == false && ret == 0)
 	{
@@ -2041,8 +2165,19 @@ UINT64 UnixGetTick64()
 	return ret;
 
 #else
-
+#ifdef	UNIX_MACOS
+	static clock_serv_t clock_serv = 0;
+	mach_timespec_t t;
+	UINT64 ret;
+	if (clock_serv == 0) {
+		host_get_clock_service(mach_host_self(), SYSTEM_CLOCK, &clock_serv);
+	}
+	clock_get_time(clock_serv, &t);
+	ret = ((UINT64)((UINT32)t.tv_sec)) * 1000LL + (UINT64)t.tv_nsec / 1000000LL;
+	return ret;
+#else
 	return TickRealtimeManual();
+#endif
 
 #endif
 }
@@ -2089,10 +2224,24 @@ void UnixSigChldHandler(int sig)
 	signal(SIGCHLD, UnixSigChldHandler);
 }
 
+// Disable core dump
+void UnixDisableCoreDump()
+{
+#ifdef	RLIMIT_CORE
+	UnixSetResourceLimit(RLIMIT_CORE, 0);
+#endif	// RLIMIT_CORE
+}
+
 // Initialize the library for UNIX
 void UnixInit()
 {
 	UNIXIO *o;
+	UINT64 max_memory = UNIX_MAX_MEMORY;
+
+	if (UnixIs64BitRlimSupported())
+	{
+		max_memory = UNIX_MAX_MEMORY_64;
+	}
 
 	UnixInitSolarisSleep();
 
@@ -2104,11 +2253,11 @@ void UnixInit()
 	current_process_id = getpid();
 
 #ifdef	RLIMIT_CORE
-	UnixSetResourceLimit(RLIMIT_CORE, UNIX_MAX_MEMORY);
+	UnixSetResourceLimit(RLIMIT_CORE, max_memory);
 #endif	// RLIMIT_CORE
 
 #ifdef	RLIMIT_DATA
-	UnixSetResourceLimit(RLIMIT_DATA, UNIX_MAX_MEMORY);
+	UnixSetResourceLimit(RLIMIT_DATA, max_memory);
 #endif	// RLIMIT_DATA
 
 #ifdef	RLIMIT_NOFILE
@@ -2120,11 +2269,11 @@ void UnixInit()
 #endif	// RLIMIT_NOFILE
 
 #ifdef	RLIMIT_STACK
-//	UnixSetResourceLimit(RLIMIT_STACK, UNIX_MAX_MEMORY);
+//	UnixSetResourceLimit(RLIMIT_STACK, max_memory);
 #endif	// RLIMIT_STACK
 
 #ifdef	RLIMIT_RSS
-	UnixSetResourceLimit(RLIMIT_RSS, UNIX_MAX_MEMORY);
+	UnixSetResourceLimit(RLIMIT_RSS, max_memory);
 #endif	// RLIMIT_RSS
 
 #ifdef	RLIMIT_LOCKS
@@ -2132,7 +2281,7 @@ void UnixInit()
 #endif	// RLIMIT_LOCKS
 
 #ifdef	RLIMIT_MEMLOCK
-	UnixSetResourceLimit(RLIMIT_MEMLOCK, UNIX_MAX_MEMORY);
+	UnixSetResourceLimit(RLIMIT_MEMLOCK, max_memory);
 #endif	// RLIMIT_MEMLOCK
 
 #ifdef	RLIMIT_NPROC
@@ -2176,27 +2325,46 @@ void UnixFree()
 }
 
 // Adjust the upper limit of resources that may be occupied
-void UnixSetResourceLimit(UINT id, UINT value)
+void UnixSetResourceLimit(UINT id, UINT64 value)
 {
 	struct rlimit t;
-	UINT hard_limit;
+	UINT64 hard_limit;
+
+	if (UnixIs64BitRlimSupported() == false)
+	{
+		if (value > (UINT64)4294967295ULL)
+		{
+			value = (UINT64)4294967295ULL;
+		}
+	}
 
 	Zero(&t, sizeof(t));
 	getrlimit(id, &t);
 
-	hard_limit = t.rlim_max;
+	hard_limit = (UINT64)t.rlim_max;
 
 	Zero(&t, sizeof(t));
-	t.rlim_cur = MIN(value, hard_limit);
-	t.rlim_max = hard_limit;
+	t.rlim_cur = (rlim_t)MIN(value, hard_limit);
+	t.rlim_max = (rlim_t)hard_limit;
 	setrlimit(id, &t);
 
 	Zero(&t, sizeof(t));
-	t.rlim_cur = value;
-	t.rlim_max = value;
+	t.rlim_cur = (rlim_t)value;
+	t.rlim_max = (rlim_t)value;
 	setrlimit(id, &t);
 }
 
+// Is the rlim_t type 64-bit?
+bool UnixIs64BitRlimSupported()
+{
+	if (sizeof(rlim_t) >= 8)
+	{
+		return true;
+	}
+
+	return false;
+}
+
 // Generate the PID file name
 void UnixGenPidFileName(char *name, UINT size)
 {
@@ -2383,6 +2551,13 @@ void UnixStartService(char *name)
 		int pid;
 		// Begin to start the service
 		UniPrint(_UU("UNIX_SVC_STARTED"), svc_title);
+
+		if (UnixGetUID() != 0)
+		{
+			// Non-root warning
+			UniPrint(_UU("UNIX_SVC_NONROOT"));
+		}
+
 		FreeSingleInstance(inst);
 
 		// Create a child process
@@ -2468,7 +2643,13 @@ void UnixStopService(char *name)
 		}
 		else
 		{
+			// SIGKILL
+			char tmp[256];
+
+			Format(tmp, sizeof(tmp), "killall -KILL %s", name);
+
 			UniPrint(_UU("UNIX_SVC_STOP_FAILED"), svc_title);
+			system(tmp);
 		}
 	}
 
@@ -2537,16 +2718,16 @@ void UnixExecService(char *name, SERVICE_FUNCTION *start, SERVICE_FUNCTION *stop
 		signal(SIGTERM, &UnixSigTermHandler);
 		while (unix_svc_terminate == false)
 		{
-#ifndef	UNIX_BSD
+#if	!(defined(UNIX_BSD) || defined(UNIX_MACOS))
 			pause();
-#else	// UNIX_BSD
+#else	// defined(UNIX_BSD) || defined(UNIX_MACOS)
 			if (UnixReadCtlFile() != saved_ctl)
 			{
 				break;
 			}
 
 			SleepThread(1394);
-#endif	// UNIX_BSD
+#endif	// defined(UNIX_BSD) || defined(UNIX_MACOS)
 		}
 
 		// Stop
@@ -2735,7 +2916,3 @@ void UnixServiceMain(int argc, char *argv[], char *name, SERVICE_FUNCTION *start
 }
 
 #endif	// UNIX
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
