@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // Mayaqua Kernel
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // Win32.c
@@ -518,7 +548,7 @@ DIRLIST *Win32EnumDirExW(wchar_t *dirname, COMPARE *compare)
 
 	UniStrCpy(tmp2, sizeof(tmp2), dirname);
 
-	if (UniStrLen(tmp2) >= 1 && tmp[UniStrLen(tmp2) - 1] == L'\\')
+	if (UniStrLen(tmp2) >= 1 && tmp2[UniStrLen(tmp2) - 1] == L'\\')
 	{
 		tmp2[UniStrLen(tmp2) - 1] = 0;
 	}
@@ -953,6 +983,24 @@ void Win32DebugAlert(char *msg)
 	MessageBox(NULL, msg, "Debug", MB_SETFOREGROUND | MB_TOPMOST | MB_SERVICE_NOTIFICATION | MB_OK | MB_ICONEXCLAMATION);
 }
 
+// Get the number of CPUs
+UINT Win32GetNumberOfCpuInner()
+{
+	UINT ret = 0;
+	SYSTEM_INFO info;
+
+	Zero(&info, sizeof(info));
+
+	GetSystemInfo(&info);
+
+	if (info.dwNumberOfProcessors >= 1 && info.dwNumberOfProcessors <= 128)
+	{
+		ret = info.dwNumberOfProcessors;
+	}
+
+	return ret;
+}
+
 // Get the OS information
 void Win32GetOsInfo(OS_INFO *info)
 {
@@ -1045,7 +1093,7 @@ bool Win32GetVersionExInternal(void *info)
 		if (os.dwPlatformId == VER_PLATFORM_WIN32_NT)
 		{
 			if ((os.dwMajorVersion == 6 && os.dwMinorVersion >= 2) ||
-				(os.dwMajorVersion == 7))
+				(os.dwMajorVersion >= 7))
 			{
 				// Windows 8 later
 				return Win32GetVersionExInternalForWindows81orLater(info);
@@ -1061,6 +1109,9 @@ bool Win32GetVersionExInternalForWindows81orLater(void *info)
 {
 	OSVERSIONINFOEXA *ex = (OSVERSIONINFOEXA *)info;
 	char *str;
+	UINT major1 = 0, major2 = 0;
+	UINT minor1 = 0, minor2 = 0;
+	UINT major = 0, minor = 0;
 	// Validate arguments
 	if (info == NULL)
 	{
@@ -1090,15 +1141,8 @@ bool Win32GetVersionExInternalForWindows81orLater(void *info)
 
 		if (t != NULL && t->NumTokens == 2)
 		{
-			UINT major = ToInt(t->Token[0]);
-			UINT minor = ToInt(t->Token[1]);
-
-			if (major >= 6)
-			{
-				// Version number acquisition success
-				ex->dwMajorVersion = major;
-				ex->dwMinorVersion = minor;
-			}
+			major1 = ToInt(t->Token[0]);
+			minor1 = ToInt(t->Token[1]);
 		}
 
 		FreeToken(t);
@@ -1106,6 +1150,32 @@ bool Win32GetVersionExInternalForWindows81orLater(void *info)
 
 	Free(str);
 
+	major2 = MsRegReadIntEx2(REG_LOCAL_MACHINE,
+		"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion",
+		"CurrentMajorVersionNumber", false, true);
+
+	minor2 = MsRegReadIntEx2(REG_LOCAL_MACHINE,
+		"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion",
+		"CurrentMinorVersionNumber", false, true);
+
+	if ((major1 * 10000 + minor1) > (major2 * 10000 + minor2))
+	{
+		major = major1;
+		minor = minor1;
+	}
+	else
+	{
+		major = major2;
+		minor = minor2;
+	}
+
+	if (major >= 6)
+	{
+		// Version number acquisition success
+		ex->dwMajorVersion = major;
+		ex->dwMinorVersion = minor;
+	}
+
 	return true;
 }
 
@@ -1377,17 +1447,30 @@ UINT Win32GetOsType()
 						return OSTYPE_WINDOWS_SERVER_81;
 					}
 				}
+				else if ((os.dwMajorVersion == 6 && os.dwMinorVersion == 4) || (os.dwMajorVersion == 10 && os.dwMinorVersion == 0))
+				{
+					if (os.wProductType == VER_NT_WORKSTATION)
+					{
+						// Windows 10
+						return OSTYPE_WINDOWS_10;
+					}
+					else
+					{
+						// Windows Server 10
+						return OSTYPE_WINDOWS_SERVER_10;
+					}
+				}
 				else
 				{
 					if (os.wProductType == VER_NT_WORKSTATION)
 					{
-						// Windows 9?
-						return OSTYPE_WINDOWS_9;
+						// Windows 11 or later
+						return OSTYPE_WINDOWS_11;
 					}
 					else
 					{
-						// Windows Server 9?
-						return OSTYPE_WINDOWS_SERVER_9;
+						// Windows Server 11 or later
+						return OSTYPE_WINDOWS_SERVER_11;
 					}
 				}
 			}
@@ -3297,7 +3380,7 @@ bool Win32InputW(wchar_t *str, UINT size)
 	{
 		DWORD read_size = 0;
 
-		if (ReadConsoleW(hstdin, str, (size - sizeof(wchar_t)), &read_size, NULL))
+		if (ReadConsoleW(hstdin, str, (size / sizeof(wchar_t)) - 1, &read_size, NULL))
 		{
 			str[read_size] = 0;
 
@@ -3457,7 +3540,3 @@ void Win32PrintToFileW(wchar_t *str)
 #endif	// WIN32
 
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
