@@ -1,20 +1,19 @@
-// SoftEther VPN Source Code
+// SoftEther VPN Source Code - Stable Edition Repository
 // SeLow: SoftEther Lightweight Network Protocol
 // 
 // SoftEther VPN Server, Client and Bridge are free software under GPLv2.
 // 
-// Copyright (c) 2012-2014 Daiyuu Nobori.
-// Copyright (c) 2012-2014 SoftEther VPN Project, University of Tsukuba, Japan.
-// Copyright (c) 2012-2014 SoftEther Corporation.
+// Copyright (c) Daiyuu Nobori.
+// Copyright (c) SoftEther VPN Project, University of Tsukuba, Japan.
+// Copyright (c) SoftEther Corporation.
 // 
 // All Rights Reserved.
 // 
 // http://www.softether.org/
 // 
-// Author: Daiyuu Nobori
+// Author: Daiyuu Nobori, Ph.D.
 // Comments: Tetsuo Sugiyama, Ph.D.
 // 
-// 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // version 2 as published by the Free Software Foundation.
@@ -55,10 +54,34 @@
 // AND FORUM NON CONVENIENS. PROCESS MAY BE SERVED ON EITHER PARTY IN
 // THE MANNER AUTHORIZED BY APPLICABLE LAW OR COURT RULE.
 // 
-// USE ONLY IN JAPAN. DO NOT USE IT IN OTHER COUNTRIES. IMPORTING THIS
-// SOFTWARE INTO OTHER COUNTRIES IS AT YOUR OWN RISK. SOME COUNTRIES
-// PROHIBIT ENCRYPTED COMMUNICATIONS. USING THIS SOFTWARE IN OTHER
-// COUNTRIES MIGHT BE RESTRICTED.
+// USE ONLY IN JAPAN. DO NOT USE THIS SOFTWARE IN ANOTHER COUNTRY UNLESS
+// YOU HAVE A CONFIRMATION THAT THIS SOFTWARE DOES NOT VIOLATE ANY
+// CRIMINAL LAWS OR CIVIL RIGHTS IN THAT PARTICULAR COUNTRY. USING THIS
+// SOFTWARE IN OTHER COUNTRIES IS COMPLETELY AT YOUR OWN RISK. THE
+// SOFTETHER VPN PROJECT HAS DEVELOPED AND DISTRIBUTED THIS SOFTWARE TO
+// COMPLY ONLY WITH THE JAPANESE LAWS AND EXISTING CIVIL RIGHTS INCLUDING
+// PATENTS WHICH ARE SUBJECTS APPLY IN JAPAN. OTHER COUNTRIES' LAWS OR
+// CIVIL RIGHTS ARE NONE OF OUR CONCERNS NOR RESPONSIBILITIES. WE HAVE
+// NEVER INVESTIGATED ANY CRIMINAL REGULATIONS, CIVIL LAWS OR
+// INTELLECTUAL PROPERTY RIGHTS INCLUDING PATENTS IN ANY OF OTHER 200+
+// COUNTRIES AND TERRITORIES. BY NATURE, THERE ARE 200+ REGIONS IN THE
+// WORLD, WITH DIFFERENT LAWS. IT IS IMPOSSIBLE TO VERIFY EVERY
+// COUNTRIES' LAWS, REGULATIONS AND CIVIL RIGHTS TO MAKE THE SOFTWARE
+// COMPLY WITH ALL COUNTRIES' LAWS BY THE PROJECT. EVEN IF YOU WILL BE
+// SUED BY A PRIVATE ENTITY OR BE DAMAGED BY A PUBLIC SERVANT IN YOUR
+// COUNTRY, THE DEVELOPERS OF THIS SOFTWARE WILL NEVER BE LIABLE TO
+// RECOVER OR COMPENSATE SUCH DAMAGES, CRIMINAL OR CIVIL
+// RESPONSIBILITIES. NOTE THAT THIS LINE IS NOT LICENSE RESTRICTION BUT
+// JUST A STATEMENT FOR WARNING AND DISCLAIMER.
+// 
+// 
+// SOURCE CODE CONTRIBUTION
+// ------------------------
+// 
+// Your contribution to SoftEther VPN Project is much appreciated.
+// Please send patches to us through GitHub.
+// Read the SoftEther VPN Patch Acceptance Policy in advance:
+// http://www.softether.org/5-download/src/9.patch
 // 
 // 
 // DEAR SECURITY EXPERTS
@@ -76,6 +99,13 @@
 // http://www.softether.org/ and ask your question on the users forum.
 // 
 // Thank you for your cooperation.
+// 
+// 
+// NO MEMORY OR RESOURCE LEAKS
+// ---------------------------
+// 
+// The memory-leaks and resource-leaks verification under the stress
+// test has been passed before release this source code.
 
 
 // SeLow.c
@@ -361,8 +391,9 @@ NTSTATUS SlDeviceOpenProc(DEVICE_OBJECT *device_object, IRP *irp)
 					p.Header.Size = NDIS_SIZEOF_NET_BUFFER_LIST_POOL_PARAMETERS_REVISION_1;
 					p.ProtocolId = NDIS_PROTOCOL_ID_DEFAULT;
 					p.fAllocateNetBuffer = true;
-					p.ContextSize = 32;
+					p.ContextSize = 32 + sizeof(UINT32) * 12;
 					p.DataSize = SL_MAX_PACKET_SIZE;
+					p.PoolTag = 'SETH';
 
 					f->NetBufferListPool = NdisAllocateNetBufferListPool(NULL, &p);
 
@@ -589,46 +620,61 @@ NTSTATUS SlDeviceReadProc(DEVICE_OBJECT *device_object, IRP *irp)
 
 			if (dst != NULL)
 			{
-				MDL *mdl;
-
-				mdl = IoAllocateMdl(dst, irp_stack->Parameters.Read.Length, false, false, NULL);
-				if (mdl != NULL)
+				// Address check
+				bool check_ok = true;
+				__try
 				{
-					MmProbeAndLockPages(mdl, KernelMode, IoWriteAccess);
+					ProbeForWrite(irp->UserBuffer, sizeof(SL_ADAPTER_INFO_LIST), 1);
+				}
+				__except (EXCEPTION_EXECUTE_HANDLER)
+				{
+					check_ok = false;
 				}
 
-				SlZero(dst, sizeof(SL_ADAPTER_INFO_LIST));
-
-				dst->Signature = SL_SIGNATURE;
-				dst->SeLowVersion = SL_VER;
-				dst->EnumCompleted = sl->IsEnumCompleted ? 8 : 1;
-
-				SlLockList(sl->AdapterList);
+				if (check_ok)
 				{
-					UINT i;
+					MDL *mdl;
 
-					dst->NumAdapters = MIN(SL_LIST_NUM(sl->AdapterList), SL_MAX_ADAPTER_INFO_LIST_ENTRY);
-
-					for (i = 0;i < dst->NumAdapters;i++)
+					mdl = IoAllocateMdl(dst, irp_stack->Parameters.Read.Length, false, false, NULL);
+					if (mdl != NULL)
 					{
-						SL_ADAPTER *a = SL_LIST_DATA(sl->AdapterList, i);
-						SL_ADAPTER_INFO *d = &dst->Adapters[i];
-
-						d->MtuSize = a->MtuSize;
-						SlCopy(d->MacAddress, a->MacAddress, 6);
-						SlCopy(d->AdapterId, a->AdapterId, sizeof(a->AdapterId));
-						strcpy(d->FriendlyName, a->FriendlyName);
+						MmProbeAndLockPages(mdl, KernelMode, IoWriteAccess);
 					}
-				}
-				SlUnlockList(sl->AdapterList);
 
-				ret_size = sizeof(SL_ADAPTER_INFO);
-				ret = STATUS_SUCCESS;
+					SlZero(dst, sizeof(SL_ADAPTER_INFO_LIST));
 
-				if (mdl != NULL)
-				{
-					MmUnlockPages(mdl);
-					IoFreeMdl(mdl);
+					dst->Signature = SL_SIGNATURE;
+					dst->SeLowVersion = SL_VER;
+					dst->EnumCompleted = sl->IsEnumCompleted ? 8 : 1;
+
+					SlLockList(sl->AdapterList);
+					{
+						UINT i;
+
+						dst->NumAdapters = MIN(SL_LIST_NUM(sl->AdapterList), SL_MAX_ADAPTER_INFO_LIST_ENTRY);
+
+						for (i = 0;i < dst->NumAdapters;i++)
+						{
+							SL_ADAPTER *a = SL_LIST_DATA(sl->AdapterList, i);
+							SL_ADAPTER_INFO *d = &dst->Adapters[i];
+
+							d->MtuSize = a->MtuSize;
+							SlCopy(d->MacAddress, a->MacAddress, 6);
+							SlCopy(d->AdapterId, a->AdapterId, sizeof(a->AdapterId));
+							strcpy(d->FriendlyName, a->FriendlyName);
+							d->SupportsVLanHw = a->SupportVLan;
+						}
+					}
+					SlUnlockList(sl->AdapterList);
+
+					ret_size = sizeof(SL_ADAPTER_INFO);
+					ret = STATUS_SUCCESS;
+
+					if (mdl != NULL)
+					{
+						MmUnlockPages(mdl);
+						IoFreeMdl(mdl);
+					}
 				}
 			}
 		}
@@ -648,75 +694,89 @@ NTSTATUS SlDeviceReadProc(DEVICE_OBJECT *device_object, IRP *irp)
 			}
 			else
 			{
-				UINT num = 0;
-				bool left = true;
-				MDL *mdl;
-				
-				mdl = IoAllocateMdl(buf, SL_EXCHANGE_BUFFER_SIZE, false, false, NULL);
-				if (mdl != NULL)
+				// Address check
+				bool check_ok = true;
+				__try
 				{
-					MmProbeAndLockPages(mdl, KernelMode, IoWriteAccess);
+					ProbeForWrite(irp->UserBuffer, SL_EXCHANGE_BUFFER_SIZE, 1);
+				}
+				__except (EXCEPTION_EXECUTE_HANDLER)
+				{
+					check_ok = false;
 				}
 
-				// Lock the receive queue
-				SlLock(f->RecvLock);
+				if (check_ok)
 				{
-					while (true)
+					UINT num = 0;
+					bool left = true;
+					MDL *mdl;
+
+					mdl = IoAllocateMdl(buf, SL_EXCHANGE_BUFFER_SIZE, false, false, NULL);
+					if (mdl != NULL)
 					{
-						SL_PACKET *q;
-						if (num >= SL_MAX_PACKET_EXCHANGE)
+						MmProbeAndLockPages(mdl, KernelMode, IoWriteAccess);
+					}
+
+					// Lock the receive queue
+					SlLock(f->RecvLock);
+					{
+						while (true)
 						{
-							if (f->RecvPacketHead == NULL)
+							SL_PACKET *q;
+							if (num >= SL_MAX_PACKET_EXCHANGE)
+							{
+								if (f->RecvPacketHead == NULL)
+								{
+									left = false;
+								}
+								break;
+							}
+							q = f->RecvPacketHead;
+							if (q != NULL)
+							{
+								f->RecvPacketHead = f->RecvPacketHead->Next;
+								q->Next = NULL;
+								f->NumRecvPackets--;
+
+								if (f->RecvPacketHead == NULL)
+								{
+									f->RecvPacketTail = NULL;
+								}
+							}
+							else
 							{
 								left = false;
+								break;
 							}
-							break;
+							SL_SIZE_OF_PACKET(buf, num) = q->Size;
+							SlCopy(SL_ADDR_OF_PACKET(buf, num), q->Data, q->Size);
+							num++;
+							SlFree(q);
 						}
-						q = f->RecvPacketHead;
-						if (q != NULL)
-						{
-							f->RecvPacketHead = f->RecvPacketHead->Next;
-							q->Next = NULL;
-							f->NumRecvPackets--;
-
-							if (f->RecvPacketHead == NULL)
-							{
-								f->RecvPacketTail = NULL;
-							}
-						}
-						else
-						{
-							left = false;
-							break;
-						}
-						SL_SIZE_OF_PACKET(buf, num) = q->Size;
-						SlCopy(SL_ADDR_OF_PACKET(buf, num), q->Data, q->Size);
-						num++;
-						SlFree(q);
 					}
-				}
-				SlUnlock(f->RecvLock);
+					SlUnlock(f->RecvLock);
 
-				if (mdl != NULL)
-				{
-					MmUnlockPages(mdl);
-					IoFreeMdl(mdl);
-				}
+					if (mdl != NULL)
+					{
+						MmUnlockPages(mdl);
+						IoFreeMdl(mdl);
+					}
 
-				SL_NUM_PACKET(buf) = num;
-				SL_LEFT_FLAG(buf) = left;
+					SL_NUM_PACKET(buf) = num;
+					SL_LEFT_FLAG(buf) = left;
 
-				if (left == false)
-				{
-					SlReset(f->Event);
-				}
-				else
-				{
-					SlSet(f->Event);
-				}
+					if (left == false)
+					{
+						SlReset(f->Event);
+					}
+					else
+					{
+						SlSet(f->Event);
+					}
 
-				ret = STATUS_SUCCESS;
-				ret_size = SL_EXCHANGE_BUFFER_SIZE;
+					ret = STATUS_SUCCESS;
+					ret_size = SL_EXCHANGE_BUFFER_SIZE;
+				}
 			}
 		}
 	}
@@ -751,69 +811,234 @@ NTSTATUS SlDeviceWriteProc(DEVICE_OBJECT *device_object, IRP *irp)
 			}
 			else
 			{
-				// Write the packet
-				MDL *mdl;
-				UINT num = SL_NUM_PACKET(buf);
-
-				mdl = IoAllocateMdl(buf, SL_EXCHANGE_BUFFER_SIZE, false, false, NULL);
-				if (mdl != NULL)
+				// Address check
+				bool check_ok = true;
+				__try
 				{
-					MmProbeAndLockPages(mdl, KernelMode, IoReadAccess);
+					ProbeForRead(irp->UserBuffer, SL_EXCHANGE_BUFFER_SIZE, 1);
+				}
+				__except (EXCEPTION_EXECUTE_HANDLER)
+				{
+					check_ok = false;
 				}
 
-				ret = true;
-				ret_size = SL_EXCHANGE_BUFFER_SIZE;
-
-				if (num >= 1 && num <= SL_MAX_PACKET_EXCHANGE)
+				if (check_ok)
 				{
-					UINT i, j;
-					NET_BUFFER_LIST *nbl_head = NULL;
-					NET_BUFFER_LIST *nbl_tail = NULL;
-					UINT num_packets = 0;
-					NDIS_HANDLE adapter_handle = NULL;
+					// Write the packet
+					MDL *mdl;
+					UINT num = SL_NUM_PACKET(buf);
 
-					SlLock(f->Adapter->Lock);
 
-					if (f->Adapter->NumPendingSendPackets <= SL_MAX_PACKET_QUEUED)
+					mdl = IoAllocateMdl(buf, SL_EXCHANGE_BUFFER_SIZE, false, false, NULL);
+					if (mdl != NULL)
 					{
-						// Admit to send only if the number of packets being transmitted does not exceed the specified limit
-						adapter_handle = f->Adapter->AdapterHandle;
+						MmProbeAndLockPages(mdl, KernelMode, IoReadAccess);
 					}
 
-					if (adapter_handle != NULL)
+					ret = true;
+					ret_size = SL_EXCHANGE_BUFFER_SIZE;
+
+					if (num >= 1 && num <= SL_MAX_PACKET_EXCHANGE)
 					{
-						// Lock the file list which opens the same adapter
-						SlLockList(dev->FileList);
-						for (j = 0;j < SL_LIST_NUM(dev->FileList);j++)
+						UINT i, j;
+						NET_BUFFER_LIST *nbl_head = NULL;
+						NET_BUFFER_LIST *nbl_tail = NULL;
+						UINT num_packets = 0;
+						NDIS_HANDLE adapter_handle = NULL;
+
+						SlLock(f->Adapter->Lock);
+
+						if (f->Adapter->NumPendingSendPackets <= SL_MAX_PACKET_QUEUED)
 						{
-							SL_FILE *other = SL_LIST_DATA(dev->FileList, j);
-
-							if (other != f)
-							{
-								// Lock the receive queue of other file lists
-								SlLock(other->RecvLock);
-
-								other->SetEventFlag = false;
-							}
+							// Admit to send only if the number of packets being transmitted does not exceed the specified limit
+							adapter_handle = f->Adapter->AdapterHandle;
 						}
 
-						for (i = 0;i < num;i++)
+						if (adapter_handle != NULL)
 						{
-							UINT packet_size = SL_SIZE_OF_PACKET(buf, i);
-							UCHAR *packet_buf;
-							NET_BUFFER_LIST *nbl = NULL;
-							bool ok = false;
+							// Lock the file list which opens the same adapter
+							SlLockList(dev->FileList);
+							for (j = 0;j < SL_LIST_NUM(dev->FileList);j++)
+							{
+								SL_FILE *other = SL_LIST_DATA(dev->FileList, j);
 
-							if (packet_size > SL_MAX_PACKET_SIZE)
-							{
-								packet_size = SL_MAX_PACKET_SIZE;
-							}
-							else if (packet_size < SL_PACKET_HEADER_SIZE)
-							{
-								packet_size = SL_PACKET_HEADER_SIZE;
+								if (other != f)
+								{
+									// Lock the receive queue of other file lists
+									SlLock(other->RecvLock);
+
+									other->SetEventFlag = false;
+								}
 							}
 
-							packet_buf = (UCHAR *)SL_ADDR_OF_PACKET(buf, i);
+							for (i = 0;i < num;i++)
+							{
+								UINT packet_size = SL_SIZE_OF_PACKET(buf, i);
+								UCHAR *packet_buf;
+								NET_BUFFER_LIST *nbl = NULL;
+								bool ok = false;
+								bool is_vlan = false;
+								UINT vlan_id = 0;
+								UINT vlan_user_priority = 0, vlan_can_format_id = 0;
+
+								if (packet_size > SL_MAX_PACKET_SIZE)
+								{
+									packet_size = SL_MAX_PACKET_SIZE;
+								}
+								else if (packet_size < SL_PACKET_HEADER_SIZE)
+								{
+									packet_size = SL_PACKET_HEADER_SIZE;
+								}
+
+								packet_buf = (UCHAR *)SL_ADDR_OF_PACKET(buf, i);
+
+								for (j = 0;j < SL_LIST_NUM(dev->FileList);j++)
+								{
+									SL_FILE *other = SL_LIST_DATA(dev->FileList, j);
+
+									if (other != f)
+									{
+										// Insert into the receive queue of the other file lists
+										if (other->NumRecvPackets < SL_MAX_PACKET_QUEUED)
+										{
+											SL_PACKET *q = SlMalloc(sizeof(SL_PACKET));
+
+											SlCopy(q->Data, packet_buf, packet_size);
+											q->Size = packet_size;
+											q->Next = NULL;
+
+											if (other->RecvPacketHead == NULL)
+											{
+												other->RecvPacketHead = q;
+											}
+											else
+											{
+												other->RecvPacketTail->Next = q;
+											}
+
+											other->RecvPacketTail = q;
+
+											other->NumRecvPackets++;
+
+											other->SetEventFlag = true;
+										}
+									}
+								}
+
+								// Allocate a new NET_BUFFER_LIST
+								if (f->NetBufferListPool != NULL)
+								{
+									nbl = NdisAllocateNetBufferList(f->NetBufferListPool, 16, 0);
+
+									if (nbl != NULL)
+									{
+										nbl->SourceHandle = adapter_handle;
+									}
+								}
+
+								if (nbl != NULL)
+								{
+									// Get the NET_BUFFER from the NET_BUFFER_LIST
+									NET_BUFFER *nb = NET_BUFFER_LIST_FIRST_NB(nbl);
+
+									NET_BUFFER_LIST_NEXT_NBL(nbl) = NULL;
+
+									// Determine if the packet is IEEE802.1Q tagged packet
+									if (dev->Adapter->SupportVLan && packet_size >= 18)
+									{
+										if (packet_buf[12] == 0x81 && packet_buf[13] == 0x00)
+										{
+											USHORT tag_us = 0;
+
+											((UCHAR *)(&tag_us))[0] = packet_buf[15];
+											((UCHAR *)(&tag_us))[1] = packet_buf[14];
+
+											vlan_id = tag_us & 0x0FFF;
+											vlan_user_priority = (tag_us >> 13) & 0x07;
+											vlan_can_format_id = (tag_us >> 12) & 0x01;
+
+											if (vlan_id != 0)
+											{
+												is_vlan = true;
+											}
+										}
+									}
+
+									if (is_vlan)
+									{
+										packet_size -= 4;
+									}
+
+									if (nb != NULL && OK(NdisRetreatNetBufferDataStart(nb, packet_size, 0, NULL)))
+									{
+										// Buffer copy
+										UCHAR *dst = NdisGetDataBuffer(nb, packet_size, NULL, 1, 0);
+
+										if (dst != NULL)
+										{
+											if (is_vlan == false)
+											{
+												SlCopy(dst, packet_buf, packet_size);
+											}
+											else
+											{
+												SlCopy(dst, packet_buf, 12);
+												SlCopy(dst + 12, packet_buf + 16, packet_size + 4 - 16);
+											}
+
+											ok = true;
+										}
+										else
+										{
+											NdisAdvanceNetBufferDataStart(nb, packet_size, false, NULL);
+										}
+									}
+								}
+
+								if (ok == false)
+								{
+									if (nbl != NULL)
+									{
+										NdisFreeNetBufferList(nbl);
+									}
+								}
+								else
+								{
+									if (nbl_head == NULL)
+									{
+										nbl_head = nbl;
+									}
+
+									if (nbl_tail != NULL)
+									{
+										NET_BUFFER_LIST_NEXT_NBL(nbl_tail) = nbl;
+									}
+
+									nbl_tail = nbl;
+
+									((void **)NET_BUFFER_LIST_CONTEXT_DATA_START(nbl))[0] = f;
+
+									if (is_vlan == false)
+									{
+										NET_BUFFER_LIST_INFO(nbl, Ieee8021QNetBufferListInfo) = NULL;
+									}
+									else
+									{
+										NDIS_NET_BUFFER_LIST_8021Q_INFO qinfo;
+
+										qinfo.Value = &(((void **)NET_BUFFER_LIST_CONTEXT_DATA_START(nbl))[1]);
+										SlZero(qinfo.Value, sizeof(UINT32) * 12);
+
+										qinfo.TagHeader.VlanId = vlan_id;
+										qinfo.TagHeader.UserPriority = vlan_user_priority;
+										qinfo.TagHeader.CanonicalFormatId = vlan_can_format_id;
+
+										NET_BUFFER_LIST_INFO(nbl, Ieee8021QNetBufferListInfo) = qinfo.Value;
+									}
+
+									num_packets++;
+								}
+							}
 
 							for (j = 0;j < SL_LIST_NUM(dev->FileList);j++)
 							{
@@ -821,138 +1046,43 @@ NTSTATUS SlDeviceWriteProc(DEVICE_OBJECT *device_object, IRP *irp)
 
 								if (other != f)
 								{
-									// Insert into the receive queue of the other file lists
-									if (other->NumRecvPackets < SL_MAX_PACKET_QUEUED)
+									// Release the receive queue of other file lists
+									SlUnlock(other->RecvLock);
+
+									// Set an event
+									if (other->SetEventFlag)
 									{
-										SL_PACKET *q = SlMalloc(sizeof(SL_PACKET));
-
-										SlCopy(q->Data, packet_buf, packet_size);
-										q->Size = packet_size;
-										q->Next = NULL;
-
-										if (other->RecvPacketHead == NULL)
-										{
-											other->RecvPacketHead = q;
-										}
-										else
-										{
-											other->RecvPacketTail->Next = q;
-										}
-
-										other->RecvPacketTail = q;
-
-										other->NumRecvPackets++;
-
-										other->SetEventFlag = true;
+										SlSet(other->Event);
 									}
 								}
 							}
+							SlUnlockList(dev->FileList);
 
-							// Allocate a new NET_BUFFER_LIST
-							if (f->NetBufferListPool != NULL)
+							if (nbl_head != NULL)
 							{
-								nbl = NdisAllocateNetBufferList(f->NetBufferListPool, 16, 0);
+								InterlockedExchangeAdd(&f->NumSendingPacketets, num_packets);
+								InterlockedExchangeAdd(&f->Adapter->NumPendingSendPackets, num_packets);
 
-								if (nbl != NULL)
-								{
-									nbl->SourceHandle = adapter_handle;
-								}
-							}
+								SlUnlock(f->Adapter->Lock);
 
-							if (nbl != NULL)
-							{
-								// Get the NET_BUFFER from the NET_BUFFER_LIST
-								NET_BUFFER *nb = NET_BUFFER_LIST_FIRST_NB(nbl);
-
-								NET_BUFFER_LIST_NEXT_NBL(nbl) = NULL;
-
-								if (nb != NULL && OK(NdisRetreatNetBufferDataStart(nb, packet_size, 0, NULL)))
-								{
-									// Buffer copy
-									UCHAR *dst = NdisGetDataBuffer(nb, packet_size, NULL, 1, 0);
-
-									if (dst != NULL)
-									{
-										SlCopy(dst, packet_buf, packet_size);
-
-										ok = true;
-									}
-									else
-									{
-										NdisAdvanceNetBufferDataStart(nb, packet_size, false, NULL);
-									}
-								}
-							}
-
-							if (ok == false)
-							{
-								if (nbl != NULL)
-								{
-									NdisFreeNetBufferList(nbl);
-								}
+								NdisSendNetBufferLists(adapter_handle, nbl_head, 0, 0);
 							}
 							else
 							{
-								if (nbl_head == NULL)
-								{
-									nbl_head = nbl;
-								}
-
-								if (nbl_tail != NULL)
-								{
-									NET_BUFFER_LIST_NEXT_NBL(nbl_tail) = nbl;
-								}
-
-								nbl_tail = nbl;
-
-								*((void **)NET_BUFFER_LIST_CONTEXT_DATA_START(nbl)) = f;
-
-								num_packets++;
+								SlUnlock(f->Adapter->Lock);
 							}
 						}
-
-						for (j = 0;j < SL_LIST_NUM(dev->FileList);j++)
-						{
-							SL_FILE *other = SL_LIST_DATA(dev->FileList, j);
-
-							if (other != f)
-							{
-								// Release the receive queue of other file lists
-								SlUnlock(other->RecvLock);
-
-								// Set an event
-								if (other->SetEventFlag)
-								{
-									SlSet(other->Event);
-								}
-							}
-						}
-						SlUnlockList(dev->FileList);
-
-						if (nbl_head != NULL)
-						{
-							InterlockedExchangeAdd(&f->NumSendingPacketets, num_packets);
-							InterlockedExchangeAdd(&f->Adapter->NumPendingSendPackets, num_packets);
-
-							SlUnlock(f->Adapter->Lock);
-
-							NdisSendNetBufferLists(adapter_handle, nbl_head, 0, 0);
-						}
 						else
 						{
 							SlUnlock(f->Adapter->Lock);
 						}
 					}
-					else
-					{
-						SlUnlock(f->Adapter->Lock);
-					}
-				}
 
-				if (mdl != NULL)
-				{
-					MmUnlockPages(mdl);
-					IoFreeMdl(mdl);
+					if (mdl != NULL)
+					{
+						MmUnlockPages(mdl);
+						IoFreeMdl(mdl);
+					}
 				}
 			}
 		}
@@ -981,17 +1111,31 @@ NTSTATUS SlDeviceIoControlProc(DEVICE_OBJECT *device_object, IRP *irp)
 		switch (irp_stack->Parameters.DeviceIoControl.IoControlCode)
 		{
 		case SL_IOCTL_GET_EVENT_NAME:
-			if (irp_stack->Parameters.DeviceIoControl.InputBufferLength >= sizeof(SL_IOCTL_EVENT_NAME))
+			if (irp_stack->Parameters.DeviceIoControl.OutputBufferLength >= sizeof(SL_IOCTL_EVENT_NAME))
 			{
 				SL_IOCTL_EVENT_NAME *t = irp->UserBuffer;
 
 				if (t != NULL)
 				{
-					strcpy(t->EventNameWin32, f->EventNameWin32);
+					// Address check
+					bool check_ok = true;
+					__try
+					{
+						ProbeForWrite(t, sizeof(SL_IOCTL_EVENT_NAME), 1);
+					}
+					__except (EXCEPTION_EXECUTE_HANDLER)
+					{
+						check_ok = false;
+					}
 
-					ret_size = sizeof(SL_IOCTL_EVENT_NAME);
+					if (check_ok)
+					{
+						strcpy(t->EventNameWin32, f->EventNameWin32);
 
-					ret = STATUS_SUCCESS;
+						ret_size = sizeof(SL_IOCTL_EVENT_NAME);
+
+						ret = STATUS_SUCCESS;
+					}
 				}
 			}
 			break;
@@ -1043,6 +1187,22 @@ NDIS_STATUS SlNdisBindAdapterExProc(NDIS_HANDLE protocol_driver_context, NDIS_HA
 		a->Lock = SlNewLock();
 		a->AdapterName = SlNewUnicodeFromUnicodeString(bind_parameters->AdapterName);
 
+/*
+		if (bind_parameters->MacOptions & NDIS_MAC_OPTION_8021Q_VLAN)
+		{
+			a->SupportVLan = true;
+		}
+
+		if (bind_parameters->TcpConnectionOffloadCapabilities != NULL)
+		{
+			if (bind_parameters->TcpConnectionOffloadCapabilities->Encapsulation & NDIS_ENCAPSULATION_IEEE_802_3_P_AND_Q ||
+				bind_parameters->TcpConnectionOffloadCapabilities->Encapsulation & NDIS_ENCAPSULATION_IEEE_802_3_P_AND_Q_IN_OOB)
+			{
+				a->SupportVLan = true;
+			}
+		}
+*/
+
 		SlCopy(a->AdapterId, a->AdapterName->String.Buffer, MIN(sizeof(a->AdapterId) - sizeof(wchar_t), a->AdapterName->String.Length));
 		SlCopy(a->AdapterId, adapter_id_tag, sizeof(adapter_id_tag) - sizeof(wchar_t));
 
@@ -1455,6 +1615,8 @@ void SlNdisReceiveNetBufferListsProc(NDIS_HANDLE protocol_binding_context, NET_B
 						USHORT tag_us;
 						is_vlan = true;
 
+						a->SupportVLan = true;
+
 						tag_us = (qinfo.TagHeader.UserPriority & 0x07 << 13) |
 							(qinfo.TagHeader.CanonicalFormatId & 0x01 << 12) |
 							(qinfo.TagHeader.VlanId & 0x0FFF);
@@ -1570,7 +1732,7 @@ void SlNdisSendNetBufferListsCompleteProc(NDIS_HANDLE protocol_binding_context,
 		}
 
 		// Get a file context
-		f = *((void **)NET_BUFFER_LIST_CONTEXT_DATA_START(nbl));
+		f = ((void **)NET_BUFFER_LIST_CONTEXT_DATA_START(nbl))[0];
 
 		nbl = NET_BUFFER_LIST_NEXT_NBL(nbl);
 		NET_BUFFER_LIST_NEXT_NBL(current_nbl) = NULL;
@@ -1601,7 +1763,7 @@ void *SlMalloc(UINT size)
 	}
 
 	// Allocate the non-paged memory
-	r = NdisAllocateMemoryWithTag(&p, size, 0);
+	r = NdisAllocateMemoryWithTag(&p, size, 'SETH');
 
 	if (NG(r))
 	{
@@ -1891,51 +2053,6 @@ NDIS_STRING *SlGetUnicode(SL_UNICODE *u)
 	return &u->String;
 }
 
-// Create a packet buffer
-SL_PACKET_BUFFER *SlNewPacketBuffer()
-{
-	SL_PACKET_BUFFER *p;
-	NET_BUFFER_LIST_POOL_PARAMETERS p1;
-
-	// Memory allocation
-	p = SlZeroMalloc(sizeof(SL_PACKET_BUFFER));
-
-	// Create a NET_BUFFER_LIST pool
-	SlZero(&p1, sizeof(p1));
-	p1.Header.Type = NDIS_OBJECT_TYPE_DEFAULT;
-	p1.Header.Revision = NET_BUFFER_LIST_POOL_PARAMETERS_REVISION_1;
-	p1.Header.Size = NDIS_SIZEOF_NET_BUFFER_LIST_POOL_PARAMETERS_REVISION_1;
-	p1.ProtocolId = NDIS_PROTOCOL_ID_DEFAULT;
-	p1.fAllocateNetBuffer = TRUE;
-	p1.DataSize = SL_MAX_PACKET_SIZE;
-
-	p->NetBufferListPool = NdisAllocateNetBufferListPool(NULL, &p1);
-
-	// Create a NET_BUFFER_LIST
-	p->NetBufferList = NdisAllocateNetBufferList(p->NetBufferListPool, 0, 0);
-
-	return p;
-}
-
-// Release the packet buffer
-void SlFreePacketBuffer(SL_PACKET_BUFFER *p)
-{
-	// Validate arguments
-	if (p == NULL)
-	{
-		return;
-	}
-
-	// Release the NET_BUFFER_LIST
-	NdisFreeNetBufferList(p->NetBufferList);
-
-	// Release the NET_BUFFER_LIST pool
-	NdisFreeNetBufferListPool(p->NetBufferListPool);
-
-	// Release the memory
-	SlFree(p);
-}
-
 // Create a list
 SL_LIST *SlNewList()
 {
@@ -2098,7 +2215,3 @@ void SlSleep(int milliSeconds)
 
 
 
-
-// Developed by SoftEther VPN Project at University of Tsukuba in Japan.
-// Department of Computer Science has dozens of overly-enthusiastic geeks.
-// Join us: http://www.tsukuba.ac.jp/english/admission/
